package com.biganto.visual.roompark.base

import android.util.Log
import com.biganto.visual.roompark.BuildConfig
import com.biganto.visual.roompark.di.dagger.AppComponent
import com.biganto.visual.roompark.di.dagger.DaggerAppComponent
import dagger.android.AndroidInjector
import dagger.android.DaggerApplication
import io.reactivex.exceptions.UndeliverableException
import io.reactivex.plugins.RxJavaPlugins
import kotlinx.io.IOException
import timber.log.Timber
import java.net.SocketException

/**
 * Created by Vladislav Bogdashkin on 03.09.2019.
 */
class RoomParkApplication : DaggerApplication() {

    companion object {
        lateinit var component: AppComponent
            private set
    }



    override fun applicationInjector(): AndroidInjector<out DaggerApplication> {
        component = DaggerAppComponent.factory()
            .create(this) as AppComponent
//            .context(this)
//            .build()
        return component
    }

    override fun onCreate() {
        super.onCreate()

        if (BuildConfig.DEBUG) Timber.plant(Timber.DebugTree())
        else Timber.plant(CrashlyticsTree())



        RxJavaPlugins.setErrorHandler { e ->
            when (e) {
                is UndeliverableException -> {
                }
                // fine, irrelevant network problem or API that throws on cancellation
                is IOException -> return@setErrorHandler
                is SocketException -> return@setErrorHandler
                // fine, some blocking code was interrupted by a dispose call
                is InterruptedException ->
                    return@setErrorHandler
                // that's likely a bug in the application
                is NullPointerException -> {
                    Thread.currentThread().uncaughtExceptionHandler
                        .uncaughtException(Thread.currentThread(), e)
                    return@setErrorHandler
                }
                is IllegalArgumentException -> {
                    Thread.currentThread().uncaughtExceptionHandler
                        .uncaughtException(Thread.currentThread(), e)
                    return@setErrorHandler
                }
                // that's a bug in RxJava or in a custom operator
                is IllegalStateException -> {
                    Thread.currentThread().uncaughtExceptionHandler
                        .uncaughtException(Thread.currentThread(), e)
                    return@setErrorHandler
                }
            }
            Timber.w(e, "Undeliverable exception received, not sure what to do")
        }
    }

}


    private class CrashlyticsTree : Timber.Tree() {
        override fun log(priority: Int, tag: String?, message: String, throwable: Throwable?) {
            if (priority == Log.VERBOSE || priority == Log.DEBUG) return

//            Crashlytics.log(priority, tag, message)
//            throwable?.let { Crashlytics.logException(it) }
        }
    }