package com.biganto.visual.roompark.conductor

import com.biganto.visual.roompark.R
import com.biganto.visual.roompark.data.service.network.NoNetworkException
import com.biganto.visual.roompark.domain.custom_exception.CustomApiException
import com.biganto.visual.roompark.util.monades.ExceptionString
import com.hannesdorfmann.mosby3.mvi.MviBasePresenter
import com.hannesdorfmann.mosby3.mvp.MvpView
import com.jakewharton.rxrelay2.PublishRelay
import io.reactivex.Observable
import timber.log.Timber

/**
 * Created by Vladislav Bogdashkin on 30.09.2019.
 */
abstract class BigantoBasePresenter<V : MvpView, VS>
: MviBasePresenter<V, VS>() {

    protected val restoreStateObservable =
        PublishRelay.create<VS>()

    fun observableParseError(t: Throwable): Observable<VS> =
        Observable.defer { Observable.just(parseError(t)) }


    abstract fun defaultErrorViewStateHandler(): (ExceptionString) -> VS

    open fun vsByCode(code: Int) =  defaultErrorViewStateHandler()
    open fun vsByThrowable(t: Throwable) =  defaultErrorViewStateHandler()

    open fun parseError(t: Throwable) : VS =
        when (t) {
            is CustomApiException -> {
                Timber.e("CustomApiException ${t.messageStringId} / ${t.customMessage}")
                parse(t)
            }
            is NoNetworkException -> {parse(t)}
            else -> {Timber.e(t);parse(t)}
        }

    private fun parse(e: CustomApiException) = onCodeReturn(e)
    private fun parse(e: NoNetworkException) = onNoNetwork(e)
    private fun parse(e: Throwable) = onRandomError(e)

    open fun onRandomError(t: Throwable): VS =
        vsByThrowable(t).invoke(
            ExceptionString(R.string.unknown_error, null)
        )

    open fun onNoNetwork(e: NoNetworkException): VS =
        vsByThrowable(e).invoke(
            ExceptionString(R.string.no_network_error, null)
        )

    private fun onCodeReturn(e: CustomApiException): VS =
        vsByCode(e.code).invoke(ExceptionString(e.messageStringId,e.customMessage))
}