package com.biganto.visual.roompark.conductor.dialogs

import android.annotation.SuppressLint
import android.content.pm.ActivityInfo
import android.graphics.Bitmap
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.ImageView
import android.widget.ProgressBar
import androidx.annotation.LayoutRes
import androidx.core.os.bundleOf
import androidx.recyclerview.widget.RecyclerView
import com.biganto.visual.roompark.R
import com.bluelinelabs.conductor.Controller
import com.bumptech.glide.Glide
import com.bumptech.glide.load.DataSource
import com.bumptech.glide.load.engine.DiskCacheStrategy
import com.bumptech.glide.load.engine.GlideException
import com.bumptech.glide.request.RequestListener
import com.bumptech.glide.request.target.Target
import com.github.chrisbanes.photoview.PhotoView
import com.google.android.material.snackbar.Snackbar
import timber.log.Timber


/**
 * Created by Vladislav Bogdashkin on 09.04.2019.
 */

internal const val  PHOTO_URL_KEY = "CHHOSE_PHOTO_LIST_KEY"


class PhotoDialogController : Controller {

    constructor(args: Bundle) : super(args)

    constructor(photoUrl: String) : super(bundleOf(PHOTO_URL_KEY to photoUrl))

    lateinit var recyclerView : RecyclerView

    @SuppressLint("SourceLockedOrientationActivity")
    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup): View {
        val view = inflater.inflate(getLayoutId(), container, false)

        val progress = view.findViewById<ProgressBar>(R.id.photo_load_progress_bar)
        progress.visibility = View.VISIBLE


        activity?.requestedOrientation = ActivityInfo.SCREEN_ORIENTATION_USER_LANDSCAPE

        args.getString(PHOTO_URL_KEY)?.let {
            val photoView = view.findViewById<PhotoView>(R.id.photo_frame)
            Glide.with(photoView)
                .asBitmap()
                .load(it)
                .diskCacheStrategy(DiskCacheStrategy.ALL)
                .addListener(object :RequestListener<Bitmap>{
                    override fun onLoadFailed(
                        e: GlideException?,
                        model: Any?,
                        target: Target<Bitmap>?,
                        isFirstResource: Boolean
                    ): Boolean {
                        progress.visibility = View.INVISIBLE
                        Snackbar.make(view,"Ошибка при загрузке!",Snackbar.LENGTH_SHORT)
                        return  false

                    }

                    override fun onResourceReady(
                        resource: Bitmap?,
                        model: Any?,
                        target: Target<Bitmap>?,
                        dataSource: DataSource?,
                        isFirstResource: Boolean
                    ): Boolean {
                        progress.visibility = View.INVISIBLE
                        return  false
                    }
                })
                .into(photoView)
        }


        view.findViewById<ImageView>(R.id.close_current_button).setOnClickListener {
            Timber.d("Clicked")
            handleBack()
        }

        return view
    }

    @LayoutRes
    fun getLayoutId() = R.layout.photo_viewer

    @SuppressLint("SourceLockedOrientationActivity")
    override fun handleBack(): Boolean {
        activity?.requestedOrientation = ActivityInfo.SCREEN_ORIENTATION_USER_PORTRAIT
        return router.popCurrentController()
    }


}
