package com.biganto.visual.roompark.data.data_provider

import com.biganto.visual.roompark.data.repository.api.room_park.IRoomParkApi
import com.biganto.visual.roompark.data.repository.db.IDb
import com.biganto.visual.roompark.data.repository.db.requrey.model.ImageAlbumJunctionEntity
import com.biganto.visual.roompark.data.repository.mapper.fromRaw
import com.biganto.visual.roompark.data.repository.mapper.fromRawList
import com.biganto.visual.roompark.domain.contract.DevProgressContract
import com.biganto.visual.roompark.domain.model.*
import io.reactivex.Observable
import io.reactivex.schedulers.Schedulers
import timber.log.Timber
import javax.inject.Inject

/**
 * Created by Vladislav Bogdashkin on 29.10.2019.
 */

//@Singleton
class AlbumsContractModule @Inject constructor(
    private val api: IRoomParkApi,
    private val db: IDb
): DevProgressContract {

    override fun getAlbumPreviews(albumId: Int): Observable<List<AlbumPhotoPreviewModel>> {
        TODO("zzz")
    }

    override fun getAlbumPreview(albumId: Int): Observable<AlbumPreviewModel> =
        db.getAlbum(albumId)
            .map (::fromEntity)

    override fun getProgressCards(): Observable<List<AlbumPreviewModel>> = fetchTopLevelAlbums()

    override fun getProgressAlbumList(albumId: Int): Observable<List<AlbumPreviewModel>> =
        fetchAlbums(albumId)


    override fun getAlbumPhoto(photoId: Int): Observable<PhotoModel> =
        db.getPhoto(photoId).map(::fromEntity)

    override fun getWebCamsList(): Observable<WebCamListModel> =
        fetchWebCams()

    override fun getWebCamStream(camId: Int): Observable<WebCamModel> {
        TODO("at current sight indeed function")
    }

    override fun getAlbumPhotoList(albumId: Int): Observable<List<PhotoModel>> =
        fetchAlbumPhotos(albumId)



    init {
        Timber.d("Devs Repository Created")
    }


    //region allAlbums
    private val fetchTopLevelAlbumsApi =
            api.getAlbums()
                .doOnNext { Timber.d("raw0 $it") }
                .map{ fromRawList(it,::fromRaw) }
                .doOnNext(db::blockingUpsert)


    private val fetchTopLevelAlbumsDb =
        db.getTopLevelAlbums()
            .doOnNext { Timber.d("top level albums count : ${it.size}") }

    private fun fetchTopLevelAlbums(): Observable<List<AlbumPreviewModel>> =
        Observable.mergeDelayError(
            arrayListOf(fetchTopLevelAlbumsDb,fetchTopLevelAlbumsApi)
        )
            .doOnNext { Timber.d("got entity $it") }
            .map { fromEntity(it,::fromEntity).sortedByDescending{ album -> album.published } }


    //endregion allAlbums

    //region concrete Albums
    private fun fetchAlbumsApi(parentAlbumId:Int) =
        api.getAlbums(parentAlbumId)
            .doOnNext { Timber.d("raw0 $it") }
            .map{ fromRawList(it,::fromRaw) }
            .doOnNext(db::blockingUpsert)
            .doOnNext {
                it.asSequence().map { album ->
                    if (db.checkIfExistsAlbumJunction(album.id, parentAlbumId) != null) {
                        return@map null
                    }
                    val entity = ImageAlbumJunctionEntity()
                    entity.setAlbumId(album.id)
                    entity.setParentId(parentAlbumId)
                    entity
                }.filterNotNull().toList().also { junctions -> db.blockingUpsert(junctions) }
            }
            .doOnNext { Timber.d("ablums ${it.size}") }
            .subscribeOn(Schedulers.io())

    private fun fetchAlbumsDb(parentAlbumId:Int) =
        db.getChildAlbums(parentAlbumId)
            .subscribeOn(Schedulers.io())

    private fun fetchAlbums(parentId:Int): Observable<List<AlbumPreviewModel>> =
        Observable.mergeDelayError(
            arrayListOf(fetchAlbumsDb(parentId),fetchAlbumsApi(parentId))
        ).map { fromEntity(it,::fromEntity) }
    //endregion concrete Albums


    //region cams
    private val fetchWebCamsApi =
        api.getWebCamsList()
            .doOnNext { Timber.d("raw0 $it") }
            .map{fromRawList(it)}
            .subscribeOn(Schedulers.io())

    private fun fetchWebCams(): Observable<WebCamListModel> = fetchWebCamsApi
    //endregion

    //region concrete Albums
    private fun fetchAlbumPhotosApi(albumId:Int) =
        api.getPhotos(albumId)
            .doOnNext { Timber.d("raw0 $it") }
            .map{ fromRawList(it,::fromRaw) }
//            .map { it }
            .doOnNext(db::blockingUpsert)
            .subscribeOn(Schedulers.io())

    private fun fetchAlbumsPhotosDb(albumId:Int) =
        db.getPhotos(albumId)
            .toList()
            .toObservable()
            .filter{it.isNotEmpty()}
            .subscribeOn(Schedulers.io())

    private fun fetchAlbumPhotos(albumId: Int): Observable<List<PhotoModel>> =
        Observable.mergeDelayError(
            arrayListOf(fetchAlbumsPhotosDb(albumId),fetchAlbumPhotosApi(albumId))
        ).map { fromEntity(it,::fromEntity) }
    //endregion concrete Albums

}

