package com.biganto.visual.roompark.data.data_provider

import com.biganto.visual.androidplayer.data.repository.local.ILocalStore
import com.biganto.visual.roompark.data.local.UserState
import com.biganto.visual.roompark.data.repository.api.room_park.IRoomParkApi
import com.biganto.visual.roompark.data.repository.db.IDb
import com.biganto.visual.roompark.data.repository.db.requrey.model.UserEntity
import com.biganto.visual.roompark.data.repository.mapper.fromRaw
import com.biganto.visual.roompark.domain.contract.AuthContract
import com.biganto.visual.roompark.domain.custom_exception.CustomApiException
import com.biganto.visual.roompark.domain.model.AuthInfoModel
import com.biganto.visual.roompark.domain.model.fromEntity
import io.reactivex.Completable
import io.reactivex.Observable
import timber.log.Timber
import javax.inject.Inject

/**
 * Created by Vladislav Bogdashkin on 29.10.2019.
 */
//


class AuthContractModule @Inject constructor(
    private val local: ILocalStore,
    private val api: IRoomParkApi,
    private val db: IDb
): AuthContract {

    init {
        Timber.d("Auth Repository Created")
    }

    override fun signOut(): Completable = local.setRecentUser(null)

    override fun signIn(email: String, password: String): Observable<AuthInfoModel> =
        api.authenticate(email,password)
            .map ( ::fromRaw )
            .flatMap{ db.upsertUser(it)  }
            .doOnNext{ Timber.d("user id: ${it.uuid}")}
            .doOnNext { local.setRecentUser(it.uuid.toString()).blockingAwait() }
            .map(::fromEntity)

    override fun validateAuthState(): Observable<Boolean> = local.recentUser()
        .map { when(it){
            is UserState.NotAuthenticated -> false
            is UserState.Authenticated -> true
            else ->false
        } }

    override fun currentUser(): Observable<UserEntity> =
        local.recentUser()
            .doOnNext { Timber.w("recentUSer: $it") }
        .flatMap{ when(it){
            is UserState.NotAuthenticated -> throw CustomApiException.NotAuthorizedException()
            is UserState.Authenticated -> db.fetchUser(it.uuid.toInt())
                .doOnError { Timber.e(it) }
                .doOnNext {u-> "got usr:$u" }
            else -> throw CustomApiException.NotAuthorizedException()
        } }

}
