package com.biganto.visual.roompark.data.data_provider

import android.app.Application
import com.biganto.visual.roompark.data.repository.api.room_park.IRoomParkApi
import com.biganto.visual.roompark.data.repository.db.IDb
import com.biganto.visual.roompark.data.repository.file.FileModule
import com.biganto.visual.roompark.domain.contract.FilesContract
import com.biganto.visual.roompark.util.extensions.folderSize
import java.io.File
import javax.inject.Inject
import kotlin.reflect.full.primaryConstructor

/**
 * Created by Vladislav Bogdashkin on 29.10.2019.
 */

//@Singleton
class FilesContractModule @Inject constructor(
    private val files: FileModule,
    private val api: IRoomParkApi,
    private val db: IDb,
    private val context: Application
): FilesContract {

    private inline fun <reified T:FileModule.FileDirectory> getDirectory(): T? =
        T::class.primaryConstructor?.call()

    private inline fun <reified T:FileModule.FileDirectory> getFile(): File {
        val d = getDirectory<T>() ?: error("Unresolved class type")
        return FileModule.getDirectory(context,d)
    }

    override fun deleteAllFiles() = files.deleteAllCacheObservable()

    override fun getPlansSize() = getFile<FileModule.FileDirectory.PlanTypeDir>().folderSize

    override fun getToursSize() = getFile<FileModule.FileDirectory.ToursDir>().folderSize

    override fun getFeedSize() = getFile<FileModule.FileDirectory.FeedsDir>().folderSize

    override fun getAlbumSize() = getFile<FileModule.FileDirectory.Albums>().folderSize

    override fun allCacheSize() = FileModule.assetsFile(context).folderSize

}

