package com.biganto.visual.roompark.data.data_provider

import com.biganto.visual.roompark.data.repository.api.room_park.IRoomParkApi
import com.biganto.visual.roompark.data.repository.db.IDb
import com.biganto.visual.roompark.data.repository.db.requrey.model.SubscriptionEntity
import com.biganto.visual.roompark.data.repository.db.requrey.model.UserEntity
import com.biganto.visual.roompark.domain.contract.SubscriptionContract
import io.reactivex.Completable
import io.reactivex.Observable
import timber.log.Timber
import javax.inject.Inject

/**
 * Created by Vladislav Bogdashkin on 25.03.2020.
 */

private const val SUBSCRIPTION_RESULT_STATUS = "OK"

class SubscriptionRepository  @Inject constructor(
    private val api: IRoomParkApi,
    private val db: IDb
): SubscriptionContract{


    override fun saveSubscribeState(sub:SubscriptionEntity): Observable<SubscriptionEntity> {
        return saveSubscribeState(
            sub.owner as UserEntity,
            sub.id,
            sub.topic,
            sub.number,
            sub.state
            )
    }

    override fun saveSubscribeState(
        userEntity: UserEntity,
        subInnerId:Int?,
        topic: String,
        topic_id: String?,
        nuewState:Boolean
    ): Observable<SubscriptionEntity> {
        var sub = subInnerId?.let {id ->
            userEntity.subscriptions?.firstOrNull { sub -> sub.id == id }
        }
        Timber.d("before nullch _subsc: $sub / ${sub?.id}")
        if (sub == null) {
            sub = SubscriptionEntity()
            sub.setOwner(userEntity)
            sub.setTopic(topic)
            sub.setNumber(topic_id)
        }
        (sub as SubscriptionEntity).setState(nuewState)
        Timber.d("before save _subsc: $sub / ${sub?.id}")
        return db.saveSubscription(sub)
    }

    override fun subscribeTopic(
        user:UserEntity,
        subInnerId:Int,
        deviceToken: String,
        topic: String,
        topic_id: String?
    ): Completable = api.subscribeTopic(
        userToken = if (topic_id!=null) TEST_DEAL_TOKEN else user.authToken,
        deviceToken = deviceToken
        ,topicName = topic
        ,topicId = topic_id)
        .flatMapCompletable {
            if (it.status == SUBSCRIPTION_RESULT_STATUS){
                saveSubscribeState(user,subInnerId,topic,topic_id,true)
                    .doOnNext { Timber.d("zzz $it") }
                    .ignoreElements()
            }
            else error("Error subscription state!")
        }


    override fun unSubscribeTopic(
    user:UserEntity,
    subInnerId:Int,
    deviceToken: String,
    topic: String,
    topic_id: String?
    ): Completable =
        api.unSuubscribeTopic(
            userToken = if (topic_id!=null) TEST_DEAL_TOKEN else user.authToken,
            deviceToken = deviceToken
            ,topicName = topic
            ,topicId = topic_id)
            .doOnNext { Timber.d("$it") }
            .flatMapCompletable  {
                if (it.status == SUBSCRIPTION_RESULT_STATUS){
                    saveSubscribeState(user,subInnerId,topic,topic_id,false)
                        .doOnNext { Timber.d("zz -z $it") }
                        .ignoreElements()
                }
                else error("Error subscription state!")
            }




}