package com.biganto.visual.roompark.data.repository.api.biganto

import com.biganto.visual.roompark.data.repository.api.biganto.raw.AppVersionRaw
import com.biganto.visual.roompark.data.repository.api.biganto.raw.TourFilesDataRaw
import com.biganto.visual.roompark.data.repository.api.biganto.raw.TourPreviewRaw
import com.google.gson.JsonArray
import io.reactivex.Flowable
import io.reactivex.Observable
import okhttp3.ResponseBody
import retrofit2.Response
import retrofit2.http.*
import java.util.*


/**
 * Created by Vladislav Bogdashkin on 13.06.2018.
 */



interface IBigantoMobileApi {

    companion object{

        //const val BASE_URL="http://local.biganto.com"
        const val BASE_URL="https://biganto.com"
        //const val API_URL="api-novus/"
        const val API_URL="api/"
        const val DELIMITER="?"
        const val CLIENT_TYPE_PARAM="client"
        const val CLIENT_VERSION_PARAM="client_version"
        const val API_VERSION_PARAM="v"
        const val AUTH_TOKEN="auth_token"

        const val LANG_PARAM="lang"
        const val DEFAULT_LANG_VALUE="en"



        const val DEFAULT_CLIENT_TYPE="androidplayer"
        //const val DEFAULT_CLIENT_TYPE="iosplayer"
        const val OLD_CLIENT_TYPE="mobileplayer"


        const val DEFAULT_CLIENT_VERSION="3.0"
        const val DEFAULT_API_VERSION="2.0"


        //region AppInfo
        const val GET_APP_VERSION="misc.appVersion"
//endregion

        //region Authentication
        const val AUTH_METHOD="users/login/"
        const val AUTH_METHOD_2_0="users.authorize"

        const val AUTH_LOGIN_PARAM="email"
        const val AUTH_PASSWORD_PARAM="password"
//endregion



        //region GetEstates
        const val GET_ESTATES_METHOD="estates.getList"
//endregion


        //region Portfolio
        const val GET_PORTFOLIO_ESTATES_METHOD="portfolio.getEstates"
        const val GET_PORTFOLIO_TOURS_METHOD="portfolio.getTours"
//endregion


        //region GetToursPreview
        const val GET_TOURS_METHOD="tours.getBadges"//"tours.getList"
        //const val PARENT_ESTATE_PARAM="id"
        const val PARENT_ESTATE_PARAM="estate_id"
        const val TOURS_BY_ID_PARAM="id"
        const val TOURS_TYPES_PARAM="types"
        const val DEFAULT_SUPPORTED_TOURS_TYPES = "virtual,real"
//endregion


        //region GetToursMeta
        const val GET_TOURS_META_METHOD="tours.getMeta"
        const val GET_TOURS_META_ID="id"
//endregion


        //region GetFiles
        const val GET_TOURS_FILES_METHOD="tours.getFiles"
        const val GET_TOURS_FILES_ID="id"
        const val GET_TOURS_FILES_RESOLUTION="resolution"
//endregion

        //region offers.GetTours
        const val OFFER_GET_TOURS_METHOD="offers.getTours"
        const val OFFER_GET_TOURS_ID="id"
        //endregion
    }

    @GET("$API_URL$GET_TOURS_METHOD$DELIMITER")
    fun getToursPreviewById(
        @Query(CLIENT_TYPE_PARAM) clientType: String = DEFAULT_CLIENT_TYPE,
        @Query(CLIENT_VERSION_PARAM) clientVersion: String = DEFAULT_CLIENT_VERSION,
        @Query(API_VERSION_PARAM) apiVersion: String = DEFAULT_API_VERSION,
        @Query(LANG_PARAM) languageCode: String = Locale.getDefault().language,
        @Query(TOURS_TYPES_PARAM) toursTypes: String = DEFAULT_SUPPORTED_TOURS_TYPES,
        @Query(TOURS_BY_ID_PARAM) ids: String
    ): Observable<Response<List<TourPreviewRaw>>>


    @GET("$API_URL$GET_TOURS_META_METHOD$DELIMITER")
    fun getTourMetaAsString(
        @Query(CLIENT_TYPE_PARAM) clientType: String = DEFAULT_CLIENT_TYPE,
        @Query(CLIENT_VERSION_PARAM) clientVersion: String = DEFAULT_CLIENT_VERSION,
        @Query(API_VERSION_PARAM) apiVersion: String = DEFAULT_API_VERSION,
        @Query(LANG_PARAM) languageCode: String = Locale.getDefault().language,
        @Query(GET_TOURS_META_ID) ids: String
    ): Observable<JsonArray>

    @GET("$API_URL$GET_TOURS_FILES_METHOD$DELIMITER")
    fun getTourFiles(
        @Query(CLIENT_TYPE_PARAM) clientType: String = DEFAULT_CLIENT_TYPE,
        @Query(CLIENT_VERSION_PARAM) clientVersion: String = DEFAULT_CLIENT_VERSION,
        @Query(API_VERSION_PARAM) apiVersion: String = DEFAULT_API_VERSION,
        @Query(LANG_PARAM) languageCode: String = Locale.getDefault().language,
        @Query(GET_TOURS_FILES_ID) ids: String,
        @Query(GET_TOURS_FILES_RESOLUTION) resolution: String
    ): Observable<Response<List<TourFilesDataRaw>>>

    @GET("$API_URL$GET_APP_VERSION$DELIMITER")
    fun getAppVersion(
        @Query(CLIENT_TYPE_PARAM) clientType: String = DEFAULT_CLIENT_TYPE,
        @Query(CLIENT_VERSION_PARAM) clientVersion: String = DEFAULT_CLIENT_VERSION,
        @Query(API_VERSION_PARAM) apiVersion: String = DEFAULT_API_VERSION,
        @Query(LANG_PARAM) languageCode: String = Locale.getDefault().language
    ): Observable<Response<AppVersionRaw>>



    @GET("$API_URL$OFFER_GET_TOURS_METHOD$DELIMITER")
    fun getOfferTours(
        @Query(CLIENT_TYPE_PARAM) clientType: String = DEFAULT_CLIENT_TYPE,
        @Query(CLIENT_VERSION_PARAM) clientVersion: String = DEFAULT_CLIENT_VERSION,
        @Query(API_VERSION_PARAM) apiVersion: String = DEFAULT_API_VERSION,
        @Query(LANG_PARAM) languageCode: String = Locale.getDefault().language,
        @Query(OFFER_GET_TOURS_ID) offerId: Int
   ): Observable<Response<Map<String,List<TourPreviewRaw>>>>

    @Streaming
    @GET
    fun requestFile(
            @HeaderMap headers: Map<String, String>?,
            @Url fileUrl: String
    ): Flowable<Response<ResponseBody>>
}