package com.biganto.visual.roompark.data.repository.api.room_park

import com.biganto.visual.roompark.data.repository.api.room_park.raw.*
import io.reactivex.Observable
import retrofit2.Response
import retrofit2.http.*


/**
 * Created by Vladislav Bogdashkin on 13.06.2018.
 */

interface IRoomParkMobileApi{

    companion object{
//        const val BASE_URL="https://stage.room-park.ru:38386"
        const val BASE_URL="https://room-park.ru"
        //const val API_URL="api-novus/"
        const val API_URL="api/"
        const val DELIMITER="?"
        const val CLIENT_TYPE_PARAM="client"
        const val CLIENT_VERSION_PARAM="client_version"
        const val API_VERSION_PARAM="v"
        const val AUTH_TOKEN="auth_token"

        const val LANG_PARAM="lang"
        const val DEFAULT_LANG_VALUE="en"

        const val DEFAULT_CLIENT_TYPE="android"
        //const val DEFAULT_CLIENT_TYPE="ios"
        const val OLD_CLIENT_TYPE="mobileplayer"

        const val DEFAULT_CLIENT_VERSION="3.0"
        const val DEFAULT_API_VERSION="2.0"

        //region AppInfo
        const val GET_APP_VERSION="misc.appVersion"
//endregion

//region Authorization
        const val AUTH_METHOD="users.authorize"
        const val EMAIL_AUTH_PARAM="email"
        const val PASSWORD_AUTH_PARAM="password"
//endregion

//region Subscription
        const val SUBSCRIBE_METHOD="users.subscribe"
        const val UNSUBSCRIBE_METHOD="users.unsubscribe"
        const val DEVICE_TOKEN_SUBSCRIBTION_PARAM="deviceToken"
        const val TOPIC_SUBSCRIBTION_PARAM="topic"
        const val TOPIC_SUBSCRIBTION_TOPIC_ID_PARAM="estate_id"
        val topicTypes = arrayOf(
            "deals",
            "progress-1",
            "progress-2",
            "progress-3",
            "progress-kindergarden",
            "progress-school",
            "progress-landscaping",
            "news",
            "blog",
            "constructing-blog"
        )
//endregion

//region GetDeals
        const val DEALS_METHOD="users.getDeals"
//endregion

//region GetFavorites
        const val FAVORITES_METHOD="users.getFavorites"
//endregion

//region GetFavorites
        const val FEEDS_METHOD="news.getFeeds"
//endregion

//region GetFavorites
        const val GET_ARTICLES_PAGE_METHOD="news.getArticlesList"
        const val ARTICLES_PAGINATION_ALIAS_PARAM="feed_alias"
        const val ARTICLES_PAGINATION_PAGE_SIZE_PARAM="pagesize"
        const val ARTICLES_PAGINATION_PAGE_PARAM="page"
//endregion

//region GetFavorites
        const val GET_ARTICLE_METHOD="news.getArticle"
        const val ARTICLE_ID_PARAM="article_id"
//endregion

//region GetAlbums
        const val ALBUMS_METHOD="progress.getAlbums"
        const val PARENT_ALBUMD_ID_PARAM="parent_id"
//endregion

//region GetAlbums
        const val PHOTOS_METHOD="progress.getPhotos"
        const val PHOTOS_ALBUMD_ID_PARAM="album_id"
//endregion

//region Get Plan Types
        const val GET_PLAN_TYPES_METHOD="estates.getPlanTypes"
        const val PLAN_TYPES_ESTATE_ID_PARAM="estate_id"
//endregion

//region Get Plan Types
        const val GET_DIRECT_PLAN_METHOD="estates.getPlan"
        const val DIRECT_PLAN_ESTATE_ID_PARAM="estate_id"
        const val DIRECT_PLAN_TYPE_PARAM="plan_id"
        const val DIRECT_PLAN_FURNITURE_PARAM="furniture"
        const val DIRECT_PLAN_SIZES_PARAM="sizes"
        const val DIRECT_PLAN_WALLS_PARAM="walls"
        const val DIRECT_PLAN_ELECTRIC_PARAM="electric"
//endregion

//region Get MultitourId
        const val GET_MULTITOUR_ID="estates.getMultiTour"
        const val MULTITOUR_BUILDING_NUMBER_PARAM="building"
        const val MULTITOUR_FLAT_NUMBER_PARAM="number"
//endregion


//region Get Estate
        const val GET_ESTATE_ID="estates.getEstate"
        const val ESTATE_BUILDING_NUMBER_PARAM="building"
        const val ESTATE_FLAT_NUMBER_PARAM="number"
//endregion

//region Get Web Cams
        const val GET_WEB_CAMS_METHOD="webcams.getCameras"
//endregion

    }


    @POST("$API_URL$AUTH_METHOD$DELIMITER")
    @FormUrlEncoded
    fun authoriz(
        @Query(CLIENT_TYPE_PARAM) clientType: String = DEFAULT_CLIENT_TYPE,
        @Query(CLIENT_VERSION_PARAM) clientVersion: String = DEFAULT_CLIENT_VERSION,
        @Query(API_VERSION_PARAM) apiVersion: String = DEFAULT_API_VERSION,
        @Field(EMAIL_AUTH_PARAM) email: String,
        @Field(PASSWORD_AUTH_PARAM) pwd: String
    ): Observable<Response<AuthRaw>>

    @POST("$API_URL$SUBSCRIBE_METHOD$DELIMITER")
    @FormUrlEncoded
    fun subscribe(
        @Query(CLIENT_TYPE_PARAM) clientType: String = DEFAULT_CLIENT_TYPE,
        @Query(CLIENT_VERSION_PARAM) clientVersion: String = DEFAULT_CLIENT_VERSION,
        @Query(API_VERSION_PARAM) apiVersion: String = DEFAULT_API_VERSION,
        @Query(AUTH_TOKEN) token: String,
        @Field(DEVICE_TOKEN_SUBSCRIBTION_PARAM) deviceToken: String,
        @Field(TOPIC_SUBSCRIBTION_PARAM) topic: String,
        @Field(TOPIC_SUBSCRIBTION_TOPIC_ID_PARAM) estateId: String?
    ): Observable<Response<StatusResponse>>

    @POST("$API_URL$UNSUBSCRIBE_METHOD$DELIMITER")
    @FormUrlEncoded
    fun unsubscribe(
        @Query(CLIENT_TYPE_PARAM) clientType: String = DEFAULT_CLIENT_TYPE,
        @Query(CLIENT_VERSION_PARAM) clientVersion: String = DEFAULT_CLIENT_VERSION,
        @Query(API_VERSION_PARAM) apiVersion: String = DEFAULT_API_VERSION,
        @Query(AUTH_TOKEN) token: String,
        @Field(DEVICE_TOKEN_SUBSCRIBTION_PARAM) deviceToken: String,
        @Field(TOPIC_SUBSCRIBTION_PARAM) topic: String,
        @Field(TOPIC_SUBSCRIBTION_TOPIC_ID_PARAM) estateId: String?
    ): Observable<Response<StatusResponse>>

    @GET("$API_URL$DEALS_METHOD$DELIMITER")
    fun getDeals(
        @Query(CLIENT_TYPE_PARAM) clientType: String = DEFAULT_CLIENT_TYPE,
        @Query(CLIENT_VERSION_PARAM) clientVersion: String = DEFAULT_CLIENT_VERSION,
        @Query(API_VERSION_PARAM) apiVersion: String = DEFAULT_API_VERSION,
        @Query(AUTH_TOKEN) token: String
    ): Observable<Response<List<DealRaw>>>

    @GET("$API_URL$FAVORITES_METHOD$DELIMITER")
    fun getFavorites(
        @Query(CLIENT_TYPE_PARAM) clientType: String = DEFAULT_CLIENT_TYPE,
        @Query(CLIENT_VERSION_PARAM) clientVersion: String = DEFAULT_CLIENT_VERSION,
        @Query(API_VERSION_PARAM) apiVersion: String = DEFAULT_API_VERSION,
        @Query(AUTH_TOKEN) token: String
    ): Observable<Response<List<EstateRaw>>>

    @GET("$API_URL$FEEDS_METHOD$DELIMITER")
    fun getFeeds(
        @Query(CLIENT_TYPE_PARAM) clientType: String = DEFAULT_CLIENT_TYPE,
        @Query(CLIENT_VERSION_PARAM) clientVersion: String = DEFAULT_CLIENT_VERSION,
        @Query(API_VERSION_PARAM) apiVersion: String = DEFAULT_API_VERSION
    ): Observable<Response<List<FeedRaw>>>

    @GET("$API_URL$GET_WEB_CAMS_METHOD$DELIMITER")
    fun getWebCams(
        @Query(CLIENT_TYPE_PARAM) clientType: String = DEFAULT_CLIENT_TYPE,
        @Query(CLIENT_VERSION_PARAM) clientVersion: String = DEFAULT_CLIENT_VERSION,
        @Query(API_VERSION_PARAM) apiVersion: String = DEFAULT_API_VERSION
    ): Observable<Response<List<WebCamRaw>>>

    @GET("$API_URL$GET_ARTICLES_PAGE_METHOD$DELIMITER")
    fun getArticlesPage(
        @Query(CLIENT_TYPE_PARAM) clientType: String = DEFAULT_CLIENT_TYPE,
        @Query(CLIENT_VERSION_PARAM) clientVersion: String = DEFAULT_CLIENT_VERSION,
        @Query(API_VERSION_PARAM) apiVersion: String = DEFAULT_API_VERSION,
        @Query(ARTICLES_PAGINATION_ALIAS_PARAM) alias: String,
        @Query(ARTICLES_PAGINATION_PAGE_SIZE_PARAM) pageSize: Int,
        @Query(ARTICLES_PAGINATION_PAGE_PARAM) page: Int
    ): Observable<Response<ArticlesListPaginationRaw>>

    @GET("$API_URL$GET_ARTICLE_METHOD$DELIMITER")
    fun getArticle(
        @Query(CLIENT_TYPE_PARAM) clientType: String = DEFAULT_CLIENT_TYPE,
        @Query(CLIENT_VERSION_PARAM) clientVersion: String = DEFAULT_CLIENT_VERSION,
        @Query(API_VERSION_PARAM) apiVersion: String = DEFAULT_API_VERSION,
        @Query(ARTICLE_ID_PARAM) id: Int
    ): Observable<Response<ArticleRaw>>

    @GET("$API_URL$ALBUMS_METHOD$DELIMITER")
    fun getAlbums(
        @Query(CLIENT_TYPE_PARAM) clientType: String = DEFAULT_CLIENT_TYPE,
        @Query(CLIENT_VERSION_PARAM) clientVersion: String = DEFAULT_CLIENT_VERSION,
        @Query(API_VERSION_PARAM) apiVersion: String = DEFAULT_API_VERSION,
        @Query(PARENT_ALBUMD_ID_PARAM) id: Int?
    ): Observable<Response<List<ImageAlbumRaw>>>

    @GET("$API_URL$PHOTOS_METHOD$DELIMITER")
    fun getPhotos(
        @Query(CLIENT_TYPE_PARAM) clientType: String = DEFAULT_CLIENT_TYPE,
        @Query(CLIENT_VERSION_PARAM) clientVersion: String = DEFAULT_CLIENT_VERSION,
        @Query(API_VERSION_PARAM) apiVersion: String = DEFAULT_API_VERSION,
        @Query(PHOTOS_ALBUMD_ID_PARAM) id: Int
    ): Observable<Response<List<GalleryImageRaw>>>

    @GET("$API_URL$GET_PLAN_TYPES_METHOD$DELIMITER")
    fun getPlanTypes(
        @Query(CLIENT_TYPE_PARAM) clientType: String = DEFAULT_CLIENT_TYPE,
        @Query(CLIENT_VERSION_PARAM) clientVersion: String = DEFAULT_CLIENT_VERSION,
        @Query(API_VERSION_PARAM) apiVersion: String = DEFAULT_API_VERSION,
        @Query(PLAN_TYPES_ESTATE_ID_PARAM) estateId: Int
    ): Observable<Response<List<PlanTypeRaw>>>

    @GET("$API_URL$GET_DIRECT_PLAN_METHOD$DELIMITER")
    fun getPlan(
        @Query(CLIENT_TYPE_PARAM) clientType: String = DEFAULT_CLIENT_TYPE,
        @Query(CLIENT_VERSION_PARAM) clientVersion: String = DEFAULT_CLIENT_VERSION,
        @Query(API_VERSION_PARAM) apiVersion: String = DEFAULT_API_VERSION,
        @Query(DIRECT_PLAN_ESTATE_ID_PARAM) estate_id:Int,
        @Query(DIRECT_PLAN_TYPE_PARAM) planType:Int,
        @Query(DIRECT_PLAN_FURNITURE_PARAM) furniture:Int,
        @Query(DIRECT_PLAN_SIZES_PARAM) sizes:Int,
        @Query(DIRECT_PLAN_WALLS_PARAM) walls:Int,
        @Query(DIRECT_PLAN_ELECTRIC_PARAM) electric:Int
    ): Observable<Response<String>>

    @Deprecated("change to estates.getEstate method as more common")
    @GET("$API_URL$GET_MULTITOUR_ID$DELIMITER")
    fun getMultiTourId(
        @Query(CLIENT_TYPE_PARAM) clientType: String = DEFAULT_CLIENT_TYPE,
        @Query(CLIENT_VERSION_PARAM) clientVersion: String = DEFAULT_CLIENT_VERSION,
        @Query(API_VERSION_PARAM) apiVersion: String = DEFAULT_API_VERSION,
        @Query(MULTITOUR_BUILDING_NUMBER_PARAM) building:Int,
        @Query(MULTITOUR_FLAT_NUMBER_PARAM) flatNumber:Int
    ): Observable<Response<Int>>

    @GET("$API_URL$GET_ESTATE_ID$DELIMITER")
    fun getEstate(
        @Query(CLIENT_TYPE_PARAM) clientType: String = DEFAULT_CLIENT_TYPE,
        @Query(CLIENT_VERSION_PARAM) clientVersion: String = DEFAULT_CLIENT_VERSION,
        @Query(API_VERSION_PARAM) apiVersion: String = DEFAULT_API_VERSION,
        @Query(ESTATE_BUILDING_NUMBER_PARAM) building:Int,
        @Query(ESTATE_FLAT_NUMBER_PARAM) flatNumber:Int
    ): Observable<Response<EstateRaw>>
}