package com.biganto.visual.roompark.data.repository.db

import com.biganto.visual.roompark.data.repository.db.requrey.RevisionString
import com.biganto.visual.roompark.data.repository.db.requrey.model.*
import com.biganto.visual.roomparkvr.data.repository.db.requery.model.DownloadState
import com.biganto.visual.roomparkvr.data.repository.db.requery.model.TourPreviewEntity
import io.reactivex.Completable
import io.reactivex.Flowable
import io.reactivex.Observable
import io.reactivex.Single
import io.requery.Persistable
import io.requery.reactivex.ReactiveResult
import io.requery.reactivex.ReactiveScalar

/**
 * Created by Vladislav Bogdashkin on 29.10.2019.
 */
interface IDb {
    fun upsertUser(entity: UserEntity): Observable<UserEntity>
    fun <T : Persistable> upsert(entity: T): Single<T>
    fun <T : List<Persistable>> upsert(entity: T): Single<Iterable<Persistable>>
    fun fetchFeeds(): Observable<FeedEntity>
    fun fetchUser(uuid: Int): Observable<UserEntity>
    fun <T : List<Persistable>> blockingUpsert(entity: T)
    fun userObservableResult(uuid: Int): Observable<ReactiveResult<UserEntity>>
    fun fetchArticles(feedAlias: String, pageSize: Int, startIndex: Int): Observable<ArticleEntity>
    fun getTopLevelAlbums(): Observable<List<ImageAlbumEntity>>
    fun getChildAlbums(parentId: Int): Observable<List<ImageAlbumEntity>>
    fun checkIfExistsAlbumJunction(albumId: Int, parentAlbumId: Int): ImageAlbumJunctionEntity?
    fun getFeed(feedAlias: String): ReactiveResult<FeedEntity>
    fun getArticle(id: Int): ReactiveResult<ArticleEntity>
    fun getPhotos(albumId: Int): Observable<GalleryPhotoEntity>
    fun getPhoto(photoId: Int): Observable<GalleryPhotoEntity>
    fun getAlbum(albumId: Int): Observable<ImageAlbumEntity>
    fun getUserFavorites(uuid: Int): Observable<EstateEntity>
    fun fetchAllUsers(): Observable<List<UserEntity>>
    fun getEstate(estateId: Int): Observable<EstateEntity>
    fun upsertEstate(entity: EstateEntity)
    fun fetchEstateByNumber(building: Int, number: String): ReactiveResult<EstateEntity>
    fun setArticleReadState(id: Int, state: Boolean): Completable
    fun setDealReadState(id: String, state: Boolean): Completable
    fun saveSubscription(subscription: SubscriptionEntity): Observable<SubscriptionEntity>
    fun getSubscription(id: Int): ReactiveResult<SubscriptionEntity>
    fun deleteTourPreview(id: String): Int?
    fun deleteTourPreview(entity: TourPreviewEntity)
    fun deleteFiles(entity: List<FileEntity>)
    fun deleteFile(entity: FileEntity)
    fun deleteTourFilesJunction(tourId: String): ReactiveScalar<Int>
    fun deleteTourFilesJunction(entity: List<TourFileJunctionEntity>)
    fun getTourFilesJunctionUniqueFiles(tourId: String): List<TourFileJunctionEntity?>
    fun getTourFilesJunction(tourId: String): ReactiveResult<TourFileJunctionEntity>
    fun upsertFileEntity(entity: List<FileEntity>): Observable<Iterable<FileEntity>>
    fun upsertFileEntity(entity: FileEntity): Observable<FileEntity>
    fun flowableFileEntityes(uris: List<RevisionString>): Flowable<FileEntity>
    fun getDownloadedSumFileEntityes(uris: List<RevisionString>): Long
    fun fetchTouresWithStates(states: List<DownloadState>): MutableList<TourPreviewEntity>
    fun fetchFileEntityes(uris: List<RevisionString>): MutableList<FileEntity>
    fun pushFileEntities(uris: List<FileEntity>): Observable<Iterable<FileEntity>>
    fun getFileEntity(uri: RevisionString): ReactiveResult<FileEntity>
    fun getTourFiles(entity: TourPreviewEntity): MutableList<TourFileJunctionEntity>
    fun getTourFiles(tourId: String): MutableList<TourFileJunctionEntity>
    fun getTourFilesObservable(tourId: String): ReactiveResult<TourFileJunctionEntity>
    fun upsertTourPreview(list: List<TourPreviewEntity>): Observable<Iterable<TourPreviewEntity>>
    fun upsertTourPreview(entity: TourPreviewEntity): Observable<TourPreviewEntity>
    fun upsertTourFileJunction(entity: List<TourFileJunctionEntity>): Observable<Iterable<TourFileJunctionEntity>>
    fun upsertTourFileJunction(entity: TourFileJunctionEntity): Observable<TourFileJunctionEntity>
    fun observableTourDownloadState(): Flowable<TourPreviewEntity>
    fun getFileEntitysOrDefault(uri: RevisionString): FileEntity
    fun getTourPreview(tourId: String): ReactiveResult<TourPreviewEntity>
    fun getTourPreviewsObservableResult(estateId: Int): Observable<ReactiveResult<TourPreviewEntity>>
    fun getEstateTourPreviews(estateId: Int): Observable<List<TourPreviewEntity>>
    fun dropFileTable(): Completable
    fun dropTourFileJuncTable(): Completable
    fun dropTourTable(): Completable
}