package com.biganto.visual.roomparkvr.data.repository.db.requery.model

import com.biganto.visual.roompark.data.repository.api.biganto.raw.TourPreviewRaw
import com.biganto.visual.roompark.data.repository.db.requrey.RevisionString
import com.biganto.visual.roompark.data.repository.db.requrey.model.Estate
import com.biganto.visual.roompark.data.repository.db.requrey.model.EstateEntity
import com.biganto.visual.roompark.data.repository.db.requrey.utils.IntListConverter
import com.biganto.visual.roompark.data.repository.db.requrey.utils.IsoDateConverter
import com.biganto.visual.roompark.data.repository.db.requrey.utils.RevisionStringConverter
import io.requery.*
import java.util.*
import kotlin.collections.ArrayList
import kotlin.math.abs

/**
 * Created by Vladislav Bogdashkin on 15.06.2018.
 */
@Entity
interface TourPreview :Persistable {
        @get:Key
        val id: String
        @get:ForeignKey(references = Estate::class)
        @get:ManyToOne
        var estate: Estate

//Obsolete
//        @get:ForeignKey(references = Tour::class)
//        @get:OneToOne(mappedBy = "id", cascade = arrayOf(CascadeAction.SAVE))
//        var tour: Tour?
        @get:Convert(IsoDateConverter::class)
        val created: Date
        @get:Convert(IsoDateConverter::class)
        val updated: Date
        val type: String
        val screen: String
        val title: String
        val preview: String?
        val hidden: Boolean

        var isDownloaded: DownloadState

        var overallSize:Long
        var tempSize:Long

        var downloadedSize:Long
        var overallFiles:Int
        var downloadedFiles:Int

        var targetResolution : Int

        @get:Convert(IntListConverter::class)
        var resolutions:ArrayList<Int>


//        @get:JunctionTable(name= "TourFilesRule")
//        @get:ManyToMany( cascade = arrayOf(CascadeAction.NONE))
//        @get:Convert(RevisionStringListConverter::class)
//        var tourFiles: ArrayList<RevisionString>?
//
        @get:Convert(RevisionStringConverter::class)
        val metaFileEntityId: RevisionString?

}


fun fromRaw(raw: TourPreviewRaw, parentId:Int, baseUrl:String, userResolution:Int):TourPreviewEntity{
        val tour=TourPreviewEntity()
        tour.setId(raw.id.toString())
        tour.setCreated(raw.created)
        tour.setUpdated(raw.updated)
        tour.setType(raw.type)
        tour.setTitle(raw.title)
        tour.setPreview("$baseUrl${raw.preview}")
        tour.setScreen("$baseUrl${raw.screen}")
        tour.setHidden(raw.hidden)
        tour.resolutions= if (raw.resolutions != null && !raw.resolutions.isNullOrEmpty()) ArrayList(raw.resolutions)
                else ArrayList(1024)
        tour.targetResolution=raw.resolutions?.nearestResolution(userResolution)?:1024
        val estate= EstateEntity()
        estate.setId(parentId)
        tour.estate=estate
        tour.isDownloaded=DownloadState.NotDownloaded
        tour.overallSize=0L
        tour.downloadedSize=0L
        tour.tempSize=0L
        tour.overallFiles=0
        tour.downloadedFiles=0

//        val tourMeta= TourEntity()
//        tourMeta.setId(raw.id.toString())
//        tour.tour=tourMeta
        return tour
}

fun List<Int>.nearestResolution(targetResolution:Int ):Int{
        var lastScore=Int.MAX_VALUE
        var lastRes=0
         for (res in this){
                 var d = targetResolution-res
                 if (d>0) d*=2
                 d=abs(d)
                 if (d<lastScore){lastScore=d;lastRes=res}
         }
//        Timber.d("Resolutions: $this  / t=$targetResolution final=$lastRes score=$lastScore")
        return  lastRes
}



fun fromRaw(raw: List<TourPreviewRaw>, parentId:Int, url:String, userResolution:Int):List<TourPreviewEntity> =List(raw.size) { index-> fromRaw(raw[index],parentId,url,userResolution) }

enum class DownloadState(val i:Int):Comparable<DownloadState>{

        NotDownloaded(0),
        MetaPreparation(1),
        DownloadQueue(2),
        Downloading(3),
        Suspended(4),
        Downloaded(5),
        Crushed(6),
        Deleting(7),
        NotSynced(20);
}
