package com.biganto.visual.roompark.domain.interactor

import com.biganto.visual.roompark.domain.model.AlbumSortedModel
import com.biganto.visual.roompark.domain.model.SubscriptionModel
import com.biganto.visual.roompark.domain.model.SubscriptionTopic
import com.biganto.visual.roompark.domain.use_case.AlbumsUseCase
import com.biganto.visual.roompark.domain.use_case.SubscriptionUseCase
import io.reactivex.Completable
import io.reactivex.Observable
import java.util.concurrent.TimeUnit
import javax.inject.Inject

/**
 * Created by Vladislav Bogdashkin on 09.10.2019.
 */

class AlbumsInteractor @Inject constructor(
    private val useCase: AlbumsUseCase,
    private val subUc: SubscriptionUseCase
) {

    fun fetchHeaderAlbums() = useCase.getProgressAlbums()

    fun fetchAlbumPhotos(parentId: Int): Observable<List<AlbumSortedModel>> =
        useCase.getChildAlbums(parentId)
            .flatMap {
                Observable.fromIterable(it).flatMap { alb ->
                    useCase.getPhotos(alb.albumId).take(1)
                        .map { photos ->
                            AlbumSortedModel(
                                alb.title,
                                alb.published,
                                alb.albumId,
                                photos
                            )
                        }
                }.toList().toObservable()
            }
            .debounce(80L, TimeUnit.MILLISECONDS) // to reduce double list draw effect

    fun getSubscriptions(topic: SubscriptionTopic) =
        subUc.getSubscriptions(topic)

    fun switchSubscription(model: SubscriptionModel, newState: Boolean): Completable =
        when(newState){
            true -> subUc.subscribeTopic(model.id,model.topic)
            false -> subUc.unSubscribeTopic(model.id,model.topic)
        }

}



