package com.biganto.visual.roompark.domain.interactor

import com.biganto.visual.roompark.domain.model.SubscriptionModel
import com.biganto.visual.roompark.domain.model.SubscriptionTopic
import com.biganto.visual.roompark.domain.use_case.FeedUseCase
import com.biganto.visual.roompark.domain.use_case.SubscriptionUseCase
import io.reactivex.Completable
import javax.inject.Inject

/**
 * Created by Vladislav Bogdashkin on 09.10.2019.
 */

class ArticlesInteractor @Inject constructor(
    private val uc: FeedUseCase,
    private val subUc: SubscriptionUseCase
) {

    fun fetchArticles(feed:String) =
        uc.getArticles(feed)

    fun fetchArticlesPage(feed:String,pageSize:Int,current:Int) =
        uc.fetchArticlesPage(feed,pageSize,current)

    fun getSubscriptions(feed: String) =
        subUc.getSubscriptions(feedSubType(feed))

    fun switchSubscription(model:SubscriptionModel,newState: Boolean): Completable =
        when(newState){
            true -> subUc.subscribeTopic(model.id,model.topic)
            false -> subUc.unSubscribeTopic(model.id,model.topic)
    }

    private fun feedSubType(feed:String) =
        when(feed){
            "news" -> SubscriptionTopic.News()
            "blog" -> SubscriptionTopic.Blog()
            "construction-blog" -> SubscriptionTopic.Construction()
            else -> error("Unknown feed type")
        }

}






























