package com.biganto.visual.roompark.domain.interactor

import com.biganto.visual.roompark.domain.model.DealModel
import com.biganto.visual.roompark.domain.model.StatusModel
import com.biganto.visual.roompark.domain.model.SubscriptionModel
import com.biganto.visual.roompark.domain.model.SubscriptionTopic
import com.biganto.visual.roompark.domain.use_case.DealseUseCase
import com.biganto.visual.roompark.domain.use_case.SubscriptionUseCase
import io.reactivex.Completable
import io.reactivex.Observable
import timber.log.Timber
import javax.inject.Inject

/**
 * Created by Vladislav Bogdashkin on 09.10.2019.
 */

class DealInteractor @Inject constructor(
    val useCase : DealseUseCase,
    val subUc : SubscriptionUseCase
){

    fun getDeal(id:String): Observable<DealModel> =
        useCase.getDeals()
            .doOnNext { Timber.d("$it") }
            .map {deals -> deals.first { it.id==id } }

    fun getStatusList() = Observable.just(statusList.sortedBy{ it.orderId })

    fun getStatusListSync() = statusList.asSequence().sortedBy{ it.orderId }.toList()

    fun setDealRead(id:String) = useCase.setDealRead(id)

    fun getSubscriptions(dealId: String) =
        subUc.getSubscriptions(SubscriptionTopic.Deals(dealId = dealId))

    fun switchSubscription(model: SubscriptionModel, newState: Boolean): Completable =
        when(newState){
            true -> subUc.subscribeTopic(model.id,model.topic)
            false -> subUc.unSubscribeTopic(model.id,model.topic)
        }

    companion object {

        val statusList = arrayListOf<StatusModel>(
            StatusModel(
                1,
                "Квартира забронирована за Вами. Скоро начнется подготовка договора",
                "Бронь"
            ),
            StatusModel(
                2,
                "Договор находится в процессе подготовки",
                "Подготовка договора"
            ),
            StatusModel(
                3,
                "Договор подготовлен и отправлен в офис продаж",
                "Договор подготовлен"
            ),
            StatusModel(
                4,
                "Договор находится в офисе продаж. Просим Вас приехать на подписание в назначенное время",
                "Договор готов к подписанию"
            ),
            StatusModel(
                5,
                "Договор передан на подписание Застройщику",
                "ДДУ подписан с покупателем"
            ),
            StatusModel(
                6,
                "Договор подписан Застройщиком",
                "Договор подписан"
            ),
            StatusModel(
                7,
                "Договор скоро будет отправлен в Росреестр",
                "Договор готовится для подачи на гос. регистрацию"
            ),
            StatusModel(
                8,
                "Договор передан в Росреестр для гос. регистрации",
                "Договор на регистрации"
            ),
            StatusModel(
                9,
                "Ваш договор успешно зарегистрирован в Росреестре",
                "Зарегистрирован"
            ),
            StatusModel(
                10,
                "Договор находится в офисе продаж. Вы можете забрать его в любое время",
                "Договор готов к выдаче"
            )
            ,
            StatusModel(
                11,
                "Договор выдан",
                "Договор выдан"
            )
        )
    }
}