package com.biganto.visual.roompark.domain.model

import android.os.Parcel
import android.os.Parcelable
import com.biganto.visual.roomparkvr.data.repository.db.requery.model.DownloadState
import com.biganto.visual.roomparkvr.data.repository.db.requery.model.TourPreviewEntity
import java.text.DecimalFormat
import java.util.*
import kotlin.math.max

/**
 * Created by Vladislav Bogdashkin on 30.05.2018.
 */

data class TourModel (
    val tour_id:String="-1",
    val created : Date=Date(),
    val updated:Date=Date(),
    val type: TourType = TourType.VIRTUAL,
    val title:String="",
    val previewUrl:String?="",
    val screen:String?=null,
    val hidden:Boolean=false,
    val metaUri:String?,
    val parentId:Int=-1,

    var downloadState: DownloadState =DownloadState.MetaPreparation,
    val downloadedSize: Long=-1L,
    val totalTourSize : Long=-1L,
    val targetResolution : Int
)   : Parcelable {

    constructor(parcel: Parcel) : this(
            parcel.readString(),
            parcel.readSerializable() as Date,
            parcel.readSerializable() as Date,
            TourType.valueOf(parcel.readString()),
            parcel.readString(),
            parcel.readString(),
            parcel.readString(),
            parcel.readByte() != 0.toByte(),
            parcel.readString(),
            parcel.readInt(),
            DownloadState.valueOf(parcel.readString()),
            parcel.readLong(),
            parcel.readLong(),
            parcel.readInt())

    val metaPredict : String
        get() = metaUri?.removeSuffix("$tour_id.json") ?: ""

    override fun writeToParcel(parcel: Parcel, flags: Int) {
        parcel.writeString(tour_id)
        parcel.writeSerializable(created)
        parcel.writeSerializable(updated)
        parcel.writeString(type.name)
        parcel.writeString(title)
        parcel.writeString(previewUrl)
        parcel.writeString(screen)
        parcel.writeByte(if (hidden) 1 else 0)
        parcel.writeString(metaUri)
        parcel.writeInt(parentId)
        parcel.writeString(downloadState.name)
        parcel.writeLong(downloadedSize)
        parcel.writeLong(totalTourSize)
        parcel.writeInt(targetResolution)
    }

    override fun describeContents(): Int {
        return 0
    }

    companion object CREATOR : Parcelable.Creator<TourModel> {
        override fun createFromParcel(parcel: Parcel): TourModel {
            return TourModel(parcel)
        }

        override fun newArray(size: Int): Array<TourModel?> {
            return arrayOfNulls(size)
        }
    }
}



fun Long.bytesToSize():String{
    if (this==-1L) return "~"
    var scale=0
    var rest:Long=this
    var least=0f
//    Timber.d("size to gb: %s",this)
    while (rest>=1024) {
        least=(rest%1024).toFloat()
        rest /= 1024
        scale++
    }
    least/=102.4f
//    least*=100f
    return when(scale)
    {
        0->"$rest.${least.format(0)}B"
        1->"$rest.${least.format(0)}Kb"
        2->"$rest.${least.format(0)}Mb"
        3->"$rest.${least.format(0)}Gb"
        4->"$rest.${least.format(0)}Tb"
        else ->"$rest${least.format(0)}Bb" //BigantoByte
    }
}

fun bytesToString(bytes:Long)=bytes.bytesToSize()

fun Float.format(fracDigits: Int): String {
    val df = DecimalFormat()
    df.maximumFractionDigits = fracDigits
    return df.format(this)
}

enum class TourType(val i:Int)
{
    VIRTUAL(0),
    REAL(1)
}

fun fromEntity(entity: TourPreviewEntity) = TourModel(
        tour_id = entity.id,
        parentId = entity.estate.id,
        created= entity.created,
        updated = entity.created,
//        updated = entity.updated,
        type = when(entity.type){
            "virtual"-> TourType.VIRTUAL
            "real"-> TourType.REAL
            else-> TourType.REAL
        },
        title = entity.title,
        previewUrl = entity.preview,
        metaUri= entity.metaFileEntityId?.uri(),
        screen = entity.screen,
        hidden = entity.hidden,
        downloadState = entity.isDownloaded,//calcDownloadState(entity.tour as TourEntity?)
        downloadedSize = entity.downloadedSize?:-1L,
        totalTourSize = max(entity.overallSize?:0L,entity.tempSize?:0L),
        targetResolution = entity.targetResolution
)

fun fromEntity(raw: List<TourPreviewEntity>):List<TourModel> =List(raw.size) { index-> fromEntity(raw[index])}


data class TourLoadProgressModel(
    val downloadState: DownloadState,
    val downloadedSize: Long=-1L,
    val totalTourSize : Long=-1L

)

