package com.biganto.visual.roompark.domain.model

import com.biganto.visual.roompark.data.repository.db.requrey.model.SubscriptionEntity

/**
 * Created by Vladislav Bogdashkin on 26.03.2020.
 */


data class SubscriptionModel(
    val topic:SubscriptionTopic,
    val id: Int,
    val state:Boolean
)

data class TitledSubscriptionModel(
    var title:String?=null,
    val subModel:SubscriptionModel
)




sealed class SubscriptionTopic(val topicName:String,val topicId:String? = null) {
    class Deals(topicName: String =TOPIC_API_NAME_DEALS,dealId:String?) : SubscriptionTopic(topicName,dealId)
    class Progress_1(topicName: String =TOPIC_API_NAME_PROGRESS_1) : SubscriptionTopic(topicName)
    class Progress_2(topicName: String =TOPIC_API_NAME_PROGRESS_2) : SubscriptionTopic(topicName)
    class Progress_3(topicName: String =TOPIC_API_NAME_PROGRESS_3) : SubscriptionTopic(topicName)
    class Progress_kindergarden(topicName: String =TOPIC_API_NAME_PROGRESS_KINDERGARDEN) : SubscriptionTopic(topicName)
    class Progress_school(topicName: String =TOPIC_API_NAME_PROGRESS_SCHOOL) : SubscriptionTopic(topicName)
    class Progress_landscaping(topicName: String =TOPIC_API_NAME_PROGRESS_LANDSCAPING) : SubscriptionTopic(topicName)
    class News(topicName: String =TOPIC_API_NAME_NEWS) : SubscriptionTopic(topicName)
    class Blog(topicName: String =TOPIC_API_NAME_BLOG) : SubscriptionTopic(topicName)
    class Construction(topicName: String = TOPIC_API_NAME_CONSCTRUCTION) : SubscriptionTopic(topicName)

    companion object {
        fun titleByTopic(topic:SubscriptionTopic) =
            when(topic){
                is Deals -> "Сделка"
                is Progress_1 -> "Ход строительства Дом №1"
                is Progress_2 -> "Ход строительства Дом №2"
                is Progress_3 -> "Ход строительства Дом №3"
                is Progress_kindergarden -> "Ход строительства \nДетский сад"
                is Progress_school -> "Ход строительства \nШкола"
                is Progress_landscaping -> "Ход строительства \nЛандшафт"
                is News -> "Новости"
                is Blog -> "Блоги"
                is Construction  -> "Строительный блог"
            }
    }

}

private const val TOPIC_API_NAME_DEALS = "deals"
private const val TOPIC_API_NAME_PROGRESS_1 = "progress-1"
private const val TOPIC_API_NAME_PROGRESS_2 = "progress-2"
private const val TOPIC_API_NAME_PROGRESS_3 = "progress-3"
private const val TOPIC_API_NAME_PROGRESS_KINDERGARDEN = "progress-kindergarden"
private const val TOPIC_API_NAME_PROGRESS_SCHOOL = "progress-school"
private const val TOPIC_API_NAME_PROGRESS_LANDSCAPING = "progress-landscaping"
private const val TOPIC_API_NAME_NEWS = "news"
private const val TOPIC_API_NAME_BLOG = "blog"
private const val TOPIC_API_NAME_CONSCTRUCTION = "construction-blog"

private fun topicFromEntity(sub: SubscriptionEntity):SubscriptionTopic =
    when(sub.topic){
        TOPIC_API_NAME_DEALS -> SubscriptionTopic.Deals(dealId = sub.number)
        TOPIC_API_NAME_PROGRESS_1 -> SubscriptionTopic.Progress_1()
        TOPIC_API_NAME_PROGRESS_2 -> SubscriptionTopic.Progress_2()
        TOPIC_API_NAME_PROGRESS_3 -> SubscriptionTopic.Progress_3()
        TOPIC_API_NAME_PROGRESS_KINDERGARDEN -> SubscriptionTopic.Progress_kindergarden()
        TOPIC_API_NAME_PROGRESS_SCHOOL -> SubscriptionTopic.Progress_school()
        TOPIC_API_NAME_PROGRESS_LANDSCAPING -> SubscriptionTopic.Progress_landscaping()
        TOPIC_API_NAME_NEWS -> SubscriptionTopic.News()
        TOPIC_API_NAME_BLOG -> SubscriptionTopic.Blog()
        TOPIC_API_NAME_CONSCTRUCTION -> SubscriptionTopic.Construction()
        else -> error("Unknown subscription type!")
    }

fun fromEntity(sub: SubscriptionEntity):SubscriptionModel =
    SubscriptionModel(topicFromEntity(sub),sub.id,sub.state)




