package com.biganto.visual.roompark.domain.use_case

import com.biganto.visual.roompark.data.data_provider.AuthContractModule
import com.biganto.visual.roompark.domain.contract.FilesContract
import com.biganto.visual.roompark.domain.contract.TourContract
import io.reactivex.Observable
import javax.inject.Inject

/**
 * Created by Vladislav Bogdashkin on 24.09.2019.
 */

class SettingsUseCase @Inject constructor(
    private val authContract: AuthContractModule,
    private val fileContract: FilesContract,
    private val tourContract: TourContract
){
    fun signOut() = authContract.signOut()

    fun clearAllCache(): Observable<Pair<Int, Int>> =
        tourContract.deleteToursDbInfo()
            .andThen(fileContract.deleteAllFiles())

    val planTypesSize
        get() = Observable.defer { Observable.just(fileContract.getPlansSize()) }

    val albumsSize
        get() = Observable.defer { Observable.just(fileContract.getAlbumSize()) }

    val feedsSize
        get() = Observable.defer { Observable.just(fileContract.getFeedSize()) }

    val toursSize
        get() = Observable.defer { Observable.just(fileContract.getToursSize()) }

    val overallSize
        get() = Observable.defer { Observable.just(fileContract.allCacheSize()) }

}