package com.biganto.visual.roompark.domain.use_case

//import io.reactivex.Observable
import com.biganto.visual.roompark.data.repository.db.requrey.model.DealEntity
import com.biganto.visual.roompark.data.repository.db.requrey.model.SubscriptionEntity
import com.biganto.visual.roompark.data.repository.db.requrey.model.UserEntity
import com.biganto.visual.roompark.domain.contract.AuthContract
import com.biganto.visual.roompark.domain.contract.DeviceUtilsContract
import com.biganto.visual.roompark.domain.contract.SubscriptionContract
import com.biganto.visual.roompark.domain.model.SubscriptionModel
import com.biganto.visual.roompark.domain.model.SubscriptionTopic
import com.biganto.visual.roompark.domain.model.TitledSubscriptionModel
import com.biganto.visual.roompark.domain.model.fromEntity
import io.reactivex.Completable
import io.reactivex.Observable
import io.reactivex.functions.BiFunction
import io.reactivex.schedulers.Schedulers
import timber.log.Timber
import javax.inject.Inject
//import io.reactivex.rxkotlin

/**
 * Created by Vladislav Bogdashkin on 01.11.2019.
 */

class SubscriptionUseCase @Inject constructor(
    private val subscription: SubscriptionContract,
    private val utils: DeviceUtilsContract,
    private val auth: AuthContract
) {
    fun subscribeTopic(subId: Int, topic: SubscriptionTopic): Completable =
        Observable.zip(auth.currentUser(), utils.getDeviceId()
            , BiFunction<UserEntity,String,SubscribeRequestModel> {
                    user, token -> SubscribeRequestModel(user,token)
        })
            .flatMapCompletable {
                subscription.subscribeTopic(
                    it.user
                    , subId
                    , it.deviceToken
                    , topic = topic.topicName,
                    topic_id = topic.topicId
                ).subscribeOn(Schedulers.io())
            }



    fun unSubscribeTopic(subId: Int, topic: SubscriptionTopic): Completable =
        Observable.zip(auth.currentUser(), utils.getDeviceId()
            , BiFunction<UserEntity,String,SubscribeRequestModel> {
                    user, token -> SubscribeRequestModel(user,token)
            })
            .flatMapCompletable {
                subscription.unSubscribeTopic(
                    it.user
                    , subId
                    , it.deviceToken
                    , topic = topic.topicName,
                    topic_id = topic.topicId
                ).subscribeOn(Schedulers.io())
            }

    fun getCurrentUserSubscriptions(): Observable<List<TitledSubscriptionModel>> =
        auth.currentUser()
            .map {user ->
                val subList = user.subscriptions?: arrayListOf()
                val list = List<TitledSubscriptionModel>(subList.size){i ->
                    val sub:SubscriptionModel = fromEntity(subList[i] as SubscriptionEntity)
                    var title = SubscriptionTopic.titleByTopic(sub.topic)
                    if (sub.topic is SubscriptionTopic.Deals){
                        val deal =
                            user.deals?.firstOrNull { d->d.id == sub.topic.topicId } as DealEntity
                        title = "$title № ${deal.estate.number}"
                    }
                    TitledSubscriptionModel(title,sub)
                }
                list
            }


    fun getSubscriptions(topic: SubscriptionTopic): Observable<SubscriptionModel> =
        auth.currentUser()
            .map {user ->
                var sub = user.subscriptions
                    ?.firstOrNull { it.topic == topic.topicName && it.number == topic.topicId }
                Timber.d("fetched topic: $sub")
                if (sub == null) {
                    sub = SubscriptionEntity()
                    sub.setOwner(user)
                    sub.setTopic(topic.topicName)
                    sub.setNumber(topic.topicId)
                    sub.setState(false)
                }
                Timber.w("sub is : $sub")
                subscription.saveSubscribeState(sub as SubscriptionEntity)
                    .doOnNext { Timber.d("zzz blocking $it") }
                    .blockingFirst()
            }
            .map(::fromEntity)
}


private data class SubscribeRequestModel(
    val user:UserEntity,
    val deviceToken:String
)




