package com.biganto.visual.roompark.player

import android.annotation.SuppressLint
import android.app.Activity
import android.content.Context
import android.content.Intent
import android.content.pm.ActivityInfo
import android.content.res.Configuration
import android.graphics.drawable.Drawable
import android.os.Bundle
import android.os.Handler
import android.view.OrientationEventListener
import android.view.View
import android.view.animation.AccelerateInterpolator
import android.view.animation.AlphaAnimation
import android.view.animation.Animation
import android.view.animation.Animation.AnimationListener
import android.widget.FrameLayout
import android.widget.ImageView
import android.widget.ProgressBar
import androidx.vectordrawable.graphics.drawable.Animatable2Compat
import androidx.vectordrawable.graphics.drawable.AnimatedVectorDrawableCompat
import com.biganto.visual.roompark.R
import com.biganto.visual.roompark.player.unity_utils.IBigantoUnityPlugin
import com.biganto.visual.roompark.player.unity_utils.LoadTourConfig
import com.biganto.visual.roompark.player.unity_utils.UnityOrientation
import com.biganto.visual.roompark.player.unity_utils.requests_raw.UPBaseRequest
import com.biganto.visual.roompark.player.unity_utils.requests_raw.UPBuildTourRequest
import com.biganto.visual.roompark.player.unity_utils.requests_raw.UPChangeOrientationRequest
import com.bumptech.glide.Glide
import com.google.gson.GsonBuilder
import com.unity3d.player.UnityPlayer
import kotlinx.android.synthetic.main.activity_biganto_player.*
//import kotlinx.android.synthetic.main.activity_biganto_player.*
import timber.log.Timber


const val PLAY_TOUR_DATA_EXTRAS="BIGANTO_PLAYER_TOUR_DATA"
const val ERROR_TOUR_BUILD_FINISH_CODE=8601
const val UNITY_SCENE_LOADED_FADE_MILLS=600L

private fun UPBaseRequest.requestToJson():String{
    return GsonBuilder().create().toJson(this, javaClass)
}

class BigantoPlayerActivity : Activity(), UnityResponseListener {

    lateinit var preview: ImageView

    lateinit var previewBackground: FrameLayout

    lateinit var biganto: ImageView

    lateinit var progress: ProgressBar

    lateinit var container: FrameLayout

    lateinit var unityContainer: FrameLayout

    private var mUnityPlayer: UnityPlayer? = null

    private var bigantoPluginListener: IBigantoUnityPlugin? = null

    private lateinit var orientationListener: OrientationListener

//    @Inject
//    lateinit var appLifeCycle: AppLifecycleListener


    private lateinit var tourObj: LoadTourConfig

    override fun onCreate(savedInstanceState: Bundle?) {
        Timber.d("CREATING PLAYER ACTIVITY")


        super.onCreate(savedInstanceState)
        orientationListener = OrientationListener(this)


//        appLifeCycle = BigantoApplication.component.appLifeCycle()
//
//        ProcessLifecycleOwner.get().lifecycle
//                .addObserver(appLifeCycle)


        setContentView(R.layout.activity_biganto_player)
        container = this.frameContainer
        unityContainer = this.unityFrameContainer
        preview = this.playerPreview
        previewBackground = this.previewBack
        biganto = this.bigantoLogo
        progress = this.loadingBar
        if (mUnityPlayer == null) {
            mUnityPlayer = UnityPlayer(this)
        }

        tourObj = intent.getParcelableExtra(PLAY_TOUR_DATA_EXTRAS)

        val animated = AnimatedVectorDrawableCompat.create(this, R.drawable.biganto_splash_anim)
        animated?.registerAnimationCallback(object : Animatable2Compat.AnimationCallback() {
            override fun onAnimationEnd(drawable: Drawable?) {
                biganto.post { animated.start() }
            }
        })
        biganto.setImageDrawable(animated)
        animated?.start()

        Glide.with(this)
                .load(tourObj.previewUri)
                //   .centerCrop()
                .into(preview)
                .also {
                    fadeOutAndHideImage(1f, 0f, 1500, biganto)
                    Handler().postDelayed(
                            {
                                fadeOutAndHideImage(0f, 1f, 1500, preview)
                            }
                            , 1200
                    )
                }
    }

    private fun fadeOutAndHideImage(from: Float, to: Float, duration: Long, img: ImageView) {
        val fadeOut = AlphaAnimation(from, to)
        fadeOut.interpolator = AccelerateInterpolator()
        fadeOut.duration = duration

        fadeOut.setAnimationListener(object : AnimationListener {
            override fun onAnimationEnd(animation: Animation) {
                if (from > to) img.visibility = View.GONE
            }

            override fun onAnimationRepeat(animation: Animation) {}
            override fun onAnimationStart(animation: Animation) {
                if (from < to) img.visibility = View.VISIBLE
            }
        })

        img.startAnimation(fadeOut)
    }

    private fun quitPlayer(){
        mUnityPlayer?.stop()
        mUnityPlayer?.quit()
    }


    override fun onBackPressed() {
        finish()
    }

    @SuppressLint("SourceLockedOrientationActivity")
    override fun onDestroy() {
        this.requestedOrientation = ActivityInfo.SCREEN_ORIENTATION_USER_PORTRAIT
        quitPlayer()
        super.onDestroy()
    }

    /**
     * invokes from unity by AndroidJavaObject mechanics
     */
    fun setUnityPlayerListener(listener: IBigantoUnityPlugin) {
        this.bigantoPluginListener = listener
//        Timber.d(" UNITY SETTING LISTENER THROGH ACTIVITY")
    }

    override fun unitySceneLoaded() {
        runOnUiThread {
            preview.visibility = View.VISIBLE
        }

        mUnityPlayer?.view?.visibility = View.VISIBLE
        bigantoPluginListener?.sendUnityTourBuildRequest(UPBuildTourRequest(tourObj).requestToJson())
    }

    override fun unityTourBuilded() {
        runOnUiThread {
            //preview.visibility = View.GONE
            fadeOutAndHideImage(1f, 0f, UNITY_SCENE_LOADED_FADE_MILLS, preview)
            progress.visibility = View.GONE
            previewBackground.visibility = View.GONE
            biganto.visibility = View.GONE
        }
        mUnityPlayer?.view?.visibility = View.VISIBLE
    }


    override fun unityTourBuildFailed() {
        Timber.d("Tour build failed! Finishing activity...")
            finish()
    }

    override fun unityTourBuildProgress(progress: Int, total: Int) {
        runOnUiThread {
            preview.visibility = View.VISIBLE
        }
        mUnityPlayer?.view?.visibility = View.VISIBLE
    }

    override fun unityCrushed(message: String) {
        runOnUiThread {
            preview.visibility = View.VISIBLE
        }
        mUnityPlayer?.view?.visibility = View.VISIBLE
    }

    override fun onNewIntent(intent: Intent) {
        Timber.d("PLAYER ACTIVITY GOT INTNET")
        // To support deep linking, we need to make sure that the client can get access to
        // the last sent intent. The clients access this through a JNI api that allows them
        // to get the intent set on launch. To update that after launch we have to manually
        // replace the intent with the one caught here.
        setIntent(intent)
    }




    // Pause Unity
    override fun onPause() {
        super.onPause()
        mUnityPlayer?.pause()
    }

    // Resume Unity
    override fun onResume() {
        Timber.d("PLAYER ACTIVITY RESUME")
        super.onResume()
        mUnityPlayer?.resume()
    }

    override fun onStart() {
        super.onStart()
        orientationListener.enable()
        if (mUnityPlayer?.view?.parent != null) return
        mUnityPlayer?.start()
        unityContainer.addView(mUnityPlayer?.view)
    }

    override fun onStop() {
        orientationListener.disable()
        bigantoPluginListener = null
        mUnityPlayer?.stop()
        super.onStop()
    }

    // Low Memory Unity
    override fun onLowMemory() {
        super.onLowMemory()
        mUnityPlayer?.lowMemory()
    }

    // Trim Memory Unity
    override fun onTrimMemory(level: Int) {
        super.onTrimMemory(level)
        if (level == TRIM_MEMORY_RUNNING_CRITICAL) {
            mUnityPlayer?.lowMemory()
        }
    }

    // This ensures the layout will be correct.
    override fun onConfigurationChanged(newConfig: Configuration) {
        super.onConfigurationChanged(newConfig)
        mUnityPlayer?.configurationChanged(newConfig)
    }

    //     Notify Unity of the focus change.
    override fun onWindowFocusChanged(hasFocus: Boolean) {
        super.onWindowFocusChanged(hasFocus)
        mUnityPlayer?.windowFocusChanged(hasFocus)
    }

//Uncomment this, if you want control backstack inside Unity engine

//    // For some reason the multiple keyevent type is not supported by the ndk.
//    // Force event injection by overriding dispatchKeyEvent().
//    override public boolean dispatchKeyEvent(KeyEvent event)
//    {
//        if (event.getAction() == KeyEvent.ACTION_MULTIPLE)
//            return mUnityPlayer.injectEvent(event);
//        return super.dispatchKeyEvent(event);
//    }
//
    // Pass any events not handled by (unfocused) views straight to UnityPlayer
//    override fun onKeyUp(keyCode: Int, event: KeyEvent): Boolean {
//        return mUnityPlayer?.injectEvent(event) ?: false;
//    }
//
//    override fun onKeyDown(keyCode: Int, event: KeyEvent): Boolean {
//        return mUnityPlayer?.injectEvent(event) ?: false
//    }
//
//    override fun onTouchEvent(event: MotionEvent): Boolean {
//        return mUnityPlayer?.injectEvent(event) ?: false
//    }
//    /*API12*/ public boolean onGenericMotionEvent(MotionEvent event)  { return mUnityPlayer.injectEvent(event); }


    inner class OrientationListener(context: Context) : OrientationEventListener(context) {
        private val ROTATION_O = 1
        private val ROTATION_90 = 2
        private val ROTATION_180 = 3
        private val ROTATION_270 = 4

        var rotation = 0


        override fun onOrientationChanged(orientation: Int) {
            var newRotation = rotation
            if ((orientation < 35 || orientation > 325) && rotation != ROTATION_O) { // PORTRAIT
                newRotation = ActivityInfo.SCREEN_ORIENTATION_PORTRAIT;
            } else if (orientation in 146..214 && rotation != ROTATION_180) { // REVERSE PORTRAIT
                newRotation = ActivityInfo.SCREEN_ORIENTATION_REVERSE_PORTRAIT;
            } else if (orientation in 56..124 && rotation != ROTATION_270) { // REVERSE LANDSCAPE
                newRotation = ActivityInfo.SCREEN_ORIENTATION_REVERSE_LANDSCAPE;
            } else if (orientation in 236..304 && rotation != ROTATION_90) { //LANDSCAPE
                newRotation = ActivityInfo.SCREEN_ORIENTATION_LANDSCAPE;
            }

            if (newRotation != rotation) {
                rotation = newRotation
//                UnityBridge.runOnUnityThread {

                Timber.d("Unity change orientation request")
//                    UnityBridge.runOnUnityThread { bigantoPluginListener?.sendUnityChangeOrientationRequest(rotation.toUnityOrientationId().requestToJson()) }
//                }
            }
        }

        fun Int.toUnityOrientationId(): UPChangeOrientationRequest {
            val orId = when (this) {
                ActivityInfo.SCREEN_ORIENTATION_PORTRAIT -> UnityOrientation.Portrait
                ActivityInfo.SCREEN_ORIENTATION_LANDSCAPE -> UnityOrientation.LandscapeLeft
                ActivityInfo.SCREEN_ORIENTATION_REVERSE_LANDSCAPE -> UnityOrientation.LandscapeRight
                ActivityInfo.SCREEN_ORIENTATION_REVERSE_PORTRAIT -> UnityOrientation.PortraitUpsideDown
                else -> UnityOrientation.Portrait
            }

            return UPChangeOrientationRequest(orId)
        }
    }
}


interface UnityResponseListener{
    fun unitySceneLoaded()
    fun unityTourBuilded()
    fun unityTourBuildProgress(progress:Int, total:Int)
    fun unityCrushed(message:String)
    fun unityTourBuildFailed()
}