package com.biganto.visual.roompark.presentation.screen.deal

import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.widget.LinearLayout
import androidx.core.os.bundleOf
import androidx.core.widget.NestedScrollView
import butterknife.BindView
import com.biganto.visual.roompark.R
import com.biganto.visual.roompark.base.HeaderToolbarModel
import com.biganto.visual.roompark.base.RoomParkApplication
import com.biganto.visual.roompark.base.RoomParkMainActivity
import com.biganto.visual.roompark.conductor.BigantoBaseController
import com.biganto.visual.roompark.conductor.dialogs.change_handler.DialogChangeHandler
import com.biganto.visual.roompark.conductor.dialogs.tour_chooser.ChooseTourDialogController
import com.biganto.visual.roompark.domain.model.DealModel
import com.biganto.visual.roompark.domain.model.typeDoubleString
import com.biganto.visual.roompark.domain.model.typeShortString
import com.biganto.visual.roompark.presentation.screen.estate.EstateScreenController
import com.biganto.visual.roompark.util.extensions.setGone
import com.biganto.visual.roompark.util.extensions.toRubly
import com.biganto.visual.roompark.util.monades.ExceptionString
import com.biganto.visual.roompark.util.view_utils.image_view.RoundedImageView
import com.biganto.visual.roompark.util.view_utils.status_progress_view.StatusProgressAnimationState
import com.biganto.visual.roompark.util.view_utils.status_progress_view.StatusProgressCeil
import com.bluelinelabs.conductor.RouterTransaction
import com.bluelinelabs.conductor.changehandler.FadeChangeHandler
import com.bumptech.glide.Glide
import com.bumptech.glide.load.engine.DiskCacheStrategy
import com.google.android.material.switchmaterial.SwitchMaterial
import com.google.android.material.textview.MaterialTextView
import com.jakewharton.rxbinding3.view.clicks
import com.jakewharton.rxbinding3.view.scrollChangeEvents
import com.jakewharton.rxbinding3.widget.checkedChanges
import io.reactivex.Observable
import io.reactivex.android.schedulers.AndroidSchedulers
import timber.log.Timber
import java.util.concurrent.TimeUnit
import javax.inject.Inject


/**
 * Created by Vladislav Bogdashkin on 30.09.2019.
 */

const val SELECTED_DEAL_ID_KEY = "SELECTED_DEAL_GUID"


class DealScreenController :
    BigantoBaseController<DealScreenViewState
        , DealScreen
        , DealScreenPresenter>
    , DealScreen {

    constructor(args: Bundle):super(args)

    constructor(id: String) : super(bundleOf(SELECTED_DEAL_ID_KEY to id))

    override fun injectDependencies() {
        getComponent()
    }

    @Inject
    override lateinit var injectedPresenter: DealScreenPresenter

    @BindView(R.id.deal_card_header)
    lateinit var dealTitle: MaterialTextView


    @BindView(R.id.deal_nestedScrollView)
    lateinit var dealContainer: NestedScrollView


    @BindView(R.id.statusContainer)
    lateinit var statusContainer: LinearLayout


    @BindView(R.id.progress_holder)
    lateinit var progressLayout: LinearLayout


    @BindView(R.id.info_ceil_1) lateinit var info1:View
    @BindView(R.id.info_ceil_2) lateinit var info2:View
    @BindView(R.id.info_ceil_3) lateinit var info3:View
    @BindView(R.id.info_ceil_4) lateinit var info4:View

    @BindView(R.id.deal_sum_value_text_view) lateinit var dealSum: MaterialTextView
    @BindView(R.id.deal_payed_value_text_view) lateinit var dealPayed: MaterialTextView
    @BindView(R.id.deal_to_pay_value_text_view) lateinit var dealSumToPay: MaterialTextView


    @BindView(R.id.flat_card_card_view)
    lateinit var toFlatView: View

    @BindView(R.id.start_tour_card)
    lateinit var startTourView: View


    @BindView(R.id.start_tour_image_view) lateinit var tourScreen: RoundedImageView


    override fun tourCardClicked(): Observable<Int> =
        tourScreen.clicks()
            .map { 1 }
            .debounce(320L, TimeUnit.MILLISECONDS)
            .observeOn(AndroidSchedulers.mainThread())


    private var servedDeal : DealModel? = null

    override fun onAttach(view: View) {
        super.onAttach(view)

        detachDisposable.addAll(
            toFlatView.clicks()
                .map { servedDeal?.estate?.id?: -1000}
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe {
                    Timber.d("got card clicked $it")
                    router.pushController(
                        RouterTransaction.with(EstateScreenController(it))
                            .popChangeHandler(FadeChangeHandler())
                            .pushChangeHandler(FadeChangeHandler())
                    )
                }
        )


        detachDisposable.add(
            dealContainer.scrollChangeEvents()
                .subscribeOn(AndroidSchedulers.mainThread())
                .subscribe {
                    if (it.scrollY > dealTitle.measuredHeight)
                        dealModel?.let { deal ->
                            toolBar.setToolbar(
                                HeaderToolbarModel(
                                    true
                                    , resources?.getString(R.string.deal_back_chevron_title)
                                    , resources?.getString(deal.estate.type.typeShortString()
                                        ,deal.estate.number)
                                    , true
                                )
                            )
                        }
                    else
                        toolBar.setToolbar(
                            HeaderToolbarModel(
                                true
                                , resources?.getString(R.string.deal_back_chevron_title)
                                , null
                                , true
                            )
                        )
                }
        )
    }

    override fun onSubscription(): Observable<Boolean> =
        toolBar.headerToolbar.findViewById<SwitchMaterial>(R.id.switch1)
            .checkedChanges()
            .skip(1) //skip init switcher check state
            .filter {
                if (!silentCheck) return@filter !silentCheck
                else silentCheck = false
                silentCheck
            }
            .doOnNext {  }
            .debounce(600L, TimeUnit.MILLISECONDS)
            .observeOn(AndroidSchedulers.mainThread())


    private fun setToolbar(){
        toolBar.setToolbar(HeaderToolbarModel(
            true
            ,resources?.getString(R.string.deal_back_chevron_title)
            ,null
            ,null)
        )


    }

    private fun bindRecycler() {
    }

    override fun onViewBound(v: View) {
        bottomNavigationController.hide()
        setToolbar()
        bindRecycler()
    }

    override fun render(viewState: DealScreenViewState) {
        super.render(viewState)
        Timber.d("Render state $viewState")
        when(viewState){
            is DealScreenViewState.Idle -> render(viewState)
            is DealScreenViewState.LoadDeal -> render(viewState)
            is DealScreenViewState.SomeError -> render(viewState)
            is DealScreenViewState.RestoreView -> render(viewState)
            is DealScreenViewState.SubscriptionStatus -> render(viewState)
            is DealScreenViewState.SubscriptionError -> render(viewState)
            is DealScreenViewState.ToursLoaded -> render(viewState)
        }
    }

    private fun render(viewState: DealScreenViewState.Idle){

    }


    private var silentCheck = false

    private fun render(viewState: DealScreenViewState.SubscriptionStatus) {
        val sw = toolBar.headerToolbar.findViewById<SwitchMaterial>(R.id.switch1)
        if (sw.isChecked != viewState.subState)
        {
            silentCheck = true
            sw.isChecked = viewState.subState
        }
    }

    private fun render(viewState: DealScreenViewState.SubscriptionError) {
        val sw = toolBar.headerToolbar.findViewById<SwitchMaterial>(R.id.switch1)
        if (sw.isChecked != viewState.subState)
        {
            silentCheck = true
            sw.isChecked = viewState.subState
        }

        if (viewState.subState)
            showError(ExceptionString(R.string.unsubscribe_error_message,null))
        else showError(ExceptionString(R.string.subscribe_error_message,null))
    }


    private fun render(viewState:DealScreenViewState.RestoreView) {

        val sw = toolBar.headerToolbar.findViewById<SwitchMaterial>(R.id.switch1)
        if (sw.isChecked != viewState.restore.sub?.state)
        {
            silentCheck = true
            sw.isChecked != sw.isChecked
        }
    }

    private fun render(viewState: DealScreenViewState.ToursLoaded) {
        router.pushController(RouterTransaction.with(
            ChooseTourDialogController(ArrayList(viewState.tours))
        )
            .popChangeHandler(DialogChangeHandler())
            .pushChangeHandler(DialogChangeHandler())
        )
    }

    private fun render(viewState: DealScreenViewState.SomeError) =
        showError(viewState.exception)


    private val View.title
        get() = this.findViewById<MaterialTextView>(R.id.info_ceil_header)
    private val View.text
        get() = this.findViewById<MaterialTextView>(R.id.info_ceil_content)

    private var dealModel:DealModel? = null

    private fun render(viewState: DealScreenViewState.LoadDeal) {
        servedDeal = viewState.estate

        startTourView.setGone(servedDeal?.estate?.multitourId == null)
        dealTitle.text = resources?.getString(
            viewState.estate.estate.type.typeDoubleString(),
            viewState.estate.estate.number
        )

        info1.title.text = resources?.getString(R.string.building)
        info1.text.text = viewState.estate.estate.commonInfo?.building.toString()

        info2.title.text = resources?.getString(R.string.section_begin)
        info2.text.text = viewState.estate.estate.commonInfo?.section_begin.toString()

        info3.title.text = resources?.getString(R.string.floor)
        info3.text.text = viewState.estate.estate.commonInfo?.floor.toString()

        info4.title.text = resources?.getString(R.string.area)
        info4.text.text =
            resources?.getString(R.string.area_value,viewState.estate.estate.commonInfo?.area)


        dealSum.text = viewState.estate.opportunitySum.toRubly()
        dealPayed.text = viewState.estate.paymentSum.toRubly()
        dealSumToPay.text = viewState.estate.amount_pay_sum.toRubly()

        viewState.estate.estate.multitourPreview?.let {
            Glide.with(tourScreen)
                .load(it)
                .diskCacheStrategy(DiskCacheStrategy.ALL)
                .into(tourScreen)
        }

        dealModel = viewState.estate

        toolBar.setToolbar(HeaderToolbarModel(
            true
            ,resources?.getString(R.string.deal_back_chevron_title)
            ,null
            ,true)
        )

        progressLayout.removeAllViews()

        viewState.statusList.forEach {

            val statusLayout = LayoutInflater.from(activity)
                .inflate(R.layout.progress_status_title_vertical
                    ,progressLayout
                    ,false)
                    as LinearLayout

            Timber.d("layouted: $statusLayout")
            val statusCeil = statusLayout.findViewById<StatusProgressCeil>(R.id.status)
            Timber.d("layouted ceail : $statusCeil")

            val position = it.orderId
            val statusCount = viewState.statusList.size
            val lastStatusPassed = viewState.estate.statusId

            statusCeil.setHasEnd(position != statusCount)
            statusCeil.setHasStart(position > 1)
            statusCeil.setIsEnabled(position <= lastStatusPassed)
            statusCeil.setNext(position < lastStatusPassed)
            if (position > lastStatusPassed)
                statusCeil.setAnimState(StatusProgressAnimationState.DISABLE)
            statusCeil.invalidate()

            val statusTitle = statusLayout.findViewById<MaterialTextView>(R.id.title)
            Timber.d("layouted statusTitle : $statusTitle")
            statusTitle.text = it.shortTitle
            progressLayout.addView(statusLayout)
        }
        progressLayout.invalidate()
    }

        private fun getComponent() = DaggerDealScreenComponent.factory()
        .create(RoomParkApplication.component,activity as RoomParkMainActivity
            , args.getString(SELECTED_DEAL_ID_KEY)?: error("Deal id not found!")
        )
        .inject(this)

    override fun getLayoutId(): Int = R.layout.deal_screen
}
