package com.biganto.visual.roompark.presentation.screen.deal

import android.content.Context
import com.biganto.visual.roompark.conductor.BigantoBasePresenter
import com.biganto.visual.roompark.domain.interactor.DealInteractor
import com.biganto.visual.roompark.domain.interactor.ToursInteractor
import com.biganto.visual.roompark.domain.model.DealModel
import com.biganto.visual.roompark.domain.model.SubscriptionModel
import com.biganto.visual.roompark.util.monades.ExceptionString
import io.reactivex.Observable
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import timber.log.Timber
import javax.inject.Inject
import javax.inject.Named

/**
 * Created by Vladislav Bogdashkin on 30.09.2019.
 */


class DealScreenPresenter @Inject constructor(
    private val interactor: DealInteractor,
    private val toursInteractor: ToursInteractor,
    private val context: Context,
    @Named(SELECTED_DEAL_ID_KEY) private val dealId:String
)
    : BigantoBasePresenter<DealScreen, DealScreenViewState>() {


    override fun defaultErrorViewStateHandler() =
        { e: ExceptionString -> DealScreenViewState.SomeError(e) }

    private var restoreModel = RestoreModel(null,null)

    override fun detachView() {
        super.detachView()
        restoreStateObservable.accept(DealScreenViewState.RestoreView(restoreModel))
    }

    override fun bindIntents() {

        val fetchDeal = interactor.getDeal(dealId)
            .doOnNext { restoreModel.deal = it }
            .map<DealScreenViewState>{ deal ->
                DealScreenViewState.LoadDeal(deal ,interactor.getStatusListSync())
            }


        val setRead = interactor.setDealRead(dealId)
            .andThen(Observable.just(DealScreenViewState.Idle()))


        val onSubChecked = intent(DealScreen::onSubscription)
            .filter { restoreModel.sub != null }
            .flatMap { newState ->
                interactor.switchSubscription(restoreModel.sub!!, newState)
                    .andThen(
                        Observable.just<DealScreenViewState>(
                            DealScreenViewState.SubscriptionStatus(newState)
                        )
                    )
                    .onErrorReturn { DealScreenViewState.SubscriptionError(!newState) }
            }

        val fetchSubscription = interactor.getSubscriptions(dealId)
            .doAfterNext { restoreModel.sub = it }
            .map<DealScreenViewState> { DealScreenViewState.SubscriptionStatus(it.state) }
            .startWith(Observable.just<DealScreenViewState>(DealScreenViewState.Idle()))


        val onStartTours = intent(DealScreen::tourCardClicked)
            .map { restoreModel.deal }
            .map{ it.estate }
            .flatMap {estate -> toursInteractor.getEstateTourList(estate)
                .map { DealScreenViewState.ToursLoaded(it) }
            }


        val state =  Observable.mergeDelayError(
            arrayListOf(
                restoreStateObservable,
                fetchDeal,
                setRead,
                onSubChecked,
                fetchSubscription,
                onStartTours
            ))
            .doOnError { Timber.e(it) }
            .onErrorReturn(::parseError)
            .subscribeOn(Schedulers.io())
            .observeOn(AndroidSchedulers.mainThread())

        subscribeViewState(state.cast(DealScreenViewState::class.java), DealScreen::render)
    }
}



data class RestoreModel(var sub: SubscriptionModel?,var deal: DealModel?)