package com.biganto.visual.roompark.presentation.screen.favorites

import com.biganto.visual.roompark.conductor.BigantoBasePresenter
import com.biganto.visual.roompark.domain.interactor.FavoritesInteractor
import com.biganto.visual.roompark.domain.interactor.ToursInteractor
import com.biganto.visual.roompark.util.monades.ExceptionString
import io.reactivex.Observable
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import timber.log.Timber
import javax.inject.Inject

/**
 * Created by Vladislav Bogdashkin on 30.09.2019.
 */


class FavoritesScreenPresenter @Inject constructor(
    private val interactor: FavoritesInteractor,
    private val toursInteractor: ToursInteractor
)
    : BigantoBasePresenter<FavoritesScreen, FavoritesScreenViewState>() {

    override fun defaultErrorViewStateHandler() =
        { e: ExceptionString -> FavoritesScreenViewState.SomeError(e) }

    override fun bindIntents() {

        val prefetchCards = interactor.getFavoritesForCurrentUser()
            .map { FavoritesScreenViewState.FavoriteEstatesLoaded(it) }


        val onStartTours = intent(FavoritesScreen::tourCardClicked)
            .flatMap {estate -> toursInteractor.getEstateTourList(estate)
                .map { FavoritesScreenViewState.ToursLoaded(it) }
            }


        val state = Observable.mergeDelayError(
            arrayListOf(
                restoreStateObservable,
                prefetchCards,
                onStartTours
            )
        )
            .doOnError { Timber.e(it) }
            .onErrorReturn(::parseError)
            .subscribeOn(Schedulers.io())
            .observeOn(AndroidSchedulers.mainThread())

        subscribeViewState(
            state.cast(FavoritesScreenViewState::class.java),
            FavoritesScreen::render
        )
    }

}