package com.biganto.visual.roompark.presentation.screen.photo.util

import android.content.Context
import android.graphics.Point
import android.view.View
import android.view.WindowManager
import butterknife.BindView
import com.biganto.visual.roompark.R
import com.biganto.visual.roompark.domain.model.PhotoModel
import com.biganto.visual.roompark.domain.model.PhotoResolutionModel
import com.biganto.visual.roompark.presentation.screen.settings.util.CommonRecyclerAdapter
import com.biganto.visual.roompark.presentation.screen.settings.util.CommonViewHolder
import com.bumptech.glide.Glide
import com.bumptech.glide.load.engine.DiskCacheStrategy
import com.github.chrisbanes.photoview.PhotoView
import com.google.android.material.textview.MaterialTextView
import timber.log.Timber
import kotlin.math.absoluteValue

/**
 * Created by Vladislav Bogdashkin on 16.10.2019.
 */


class PhotosAdapter : CommonRecyclerAdapter<PhotosViewHolder, PhotoModel>() {

    val photosPreviewList:List<String> get() {
         val ret =  list.map {  it.resolutionList.lowelest()?.url?: error("No res urls!")}.toList()
        Timber.d("$ret")
        return ret
    }

    fun getItem(index:Int) = list[index]

    fun indexById(id:Int):Int = list.indexOfFirst { it.photoId == id }

    fun getCurrentItem(pos:Int) = list[pos]


    override val vhKlazz = PhotosViewHolder::class

    override fun getVhLayout(): Int = R.layout.photo_page_viewholder
}

class PhotosViewHolder(itemView: View) : CommonViewHolder<PhotoModel>(itemView) {

    @BindView(R.id.photo_view)
    lateinit var photoPreview: PhotoView

    @BindView(R.id.photo_description)
    lateinit var description: MaterialTextView

    private val windowSize by lazy {
        val pSize: Point = Point()
        (itemView.context.getSystemService(Context.WINDOW_SERVICE) as WindowManager)
            .defaultDisplay.getSize(pSize)
        pSize
    }

    override fun onViewBound(model: PhotoModel) {

        description.visibility = if (model.description != null) View.VISIBLE else View.GONE
        model.description?.let { description.text = it }

        model.resolutionList
            .minBy {
                ((windowSize.x - it.resWidth + .1f).absoluteValue
                        + (windowSize.y - it.resHeight + .1f).absoluteValue
                        )
            }
            ?.let { photo ->
                Timber.d("val: ${photo.resWidth} : ${photo.resHeight}")
                Glide.with(photoPreview)
                    .load(photo.url)
                    .thumbnail(model.resolutionList.lowelest()?.let { thumb ->
                        Glide.with(photoPreview).load(thumb.url).diskCacheStrategy(DiskCacheStrategy.ALL)
                            .centerCrop()
                    })
                    .centerCrop()
                    .diskCacheStrategy(DiskCacheStrategy.ALL)
                    .into(photoPreview)
            }

    }
}


private fun List<PhotoResolutionModel>.lowelest() =
    this.minBy { it.resWidth * it.resHeight }