package com.biganto.visual.roompark.presentation.screen.to_flat

import com.biganto.visual.roompark.R
import com.biganto.visual.roompark.conductor.BigantoBasePresenter
import com.biganto.visual.roompark.domain.interactor.FindFlatInteractor
import com.biganto.visual.roompark.util.monades.ExceptionString
import io.reactivex.Observable
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import timber.log.Timber
import javax.inject.Inject

/**
 * Created by Vladislav Bogdashkin on 30.09.2019.
 */


class FindFlatScreenPresenter @Inject constructor(
    private val interactor: FindFlatInteractor
)
    : BigantoBasePresenter<FindFlatScreen, FindFlatScreenViewState>() {

    override fun defaultErrorViewStateHandler() =
        {e: ExceptionString -> FindFlatScreenViewState.SomeError(e)}


    //estateId
    private var recentFlat:Int? =null

    private var restoreModel = RestoreModel(-1,-1)

    override fun detachView() {
        super.detachView()
        restoreStateObservable.accept(FindFlatScreenViewState.RestoreView(restoreModel))
    }

    override fun vsByCode(code: Int): (ExceptionString) -> FindFlatScreenViewState =
     when (code) {
            304 -> {message:ExceptionString ->FindFlatScreenViewState.FlatNotFound(message)}
            else -> super.vsByCode(code)
        }

    override fun bindIntents() {

        val getFlatIntent = intent(FindFlatScreen::getFlat)
            .doOnNext { restoreModel.building = it.building;restoreModel.number = it.number }
            .doOnNext{recentFlat=null}
            .doOnNext{Timber.d(" flat is $it")}
            .flatMap { request ->
                interactor.getFlat(request.building,request.number)
                    .doOnNext { recentFlat=it }
                    .map<FindFlatScreenViewState> {FindFlatScreenViewState.FlatFounded()}
                    .onErrorReturn (::parseError)
            }
            .startWith(Observable.just(FindFlatScreenViewState.RequstFlatProgress()))

        val startFlatIntent = intent(FindFlatScreen::openFlat)
            .doOnNext { restoreModel.building = it.building;restoreModel.number = it.number }
            .map {
                if (recentFlat != null) {FindFlatScreenViewState.StartFlat(recentFlat as Int)}
                else FindFlatScreenViewState.FlatNotFound(
                    ExceptionString(R.string.flat_not_found,null)
                )
            }



        val state = restoreStateObservable
            .mergeWith(getFlatIntent)
            .mergeWith(startFlatIntent)
            .doOnError { Timber.e(it) }
            .subscribeOn(Schedulers.io())
            .observeOn(AndroidSchedulers.mainThread())

        subscribeViewState(state.cast(FindFlatScreenViewState::class.java), FindFlatScreen::render)
    }
}

data class RestoreModel(
    var building:Int,var number:Int)
