package com.biganto.visual.roompark.util.view_utils.snackbar

import android.app.Activity
import android.view.View
import androidx.annotation.StringRes
import androidx.core.content.ContextCompat
import com.biganto.visual.roompark.R
import com.biganto.visual.roompark.base.IBottomNavigation
import com.biganto.visual.roompark.util.view_utils.snackbar.SnackBarMessageType.*
import com.google.android.material.snackbar.Snackbar
import timber.log.Timber
import javax.inject.Inject

/**
 * Created by Vladislav Bogdashkin on 06.05.2019.
 */


class SnackBarProvider @Inject constructor(val activity: Activity) : ISnackBarProvider {

    override val isRootBounded: Boolean
        get() = rootView != null

    override val snackBar: Snackbar?
        get() = snack

    private var snack: Snackbar? = null

    private var rootView: View? = null

    private val parentView: View
        get()  = rootView
                ?: activity.window.decorView.rootView

    private fun color(type: SnackBarMessageType) =
            ContextCompat.getColor(activity,
                    when (type) {
                        DEFAULT -> R.color.colorAccent
                        ERROR -> R.color.colorAccent
                        ATTENTION -> R.color.colorAccent
                        OK -> R.color.colorAccent
                    })

    private fun actionText(type: SnackBarMessageType) =
            activity.getString(
                    when (type) {
                        DEFAULT -> R.string.snackbar_dismiss_button_default
                        ERROR -> R.string.snackbar_dismiss_button_error
                        ATTENTION -> R.string.snackbar_dismiss_button_attention
                        OK -> R.string.snackbar_dismiss_button_ok
                    })


    override fun bindRootView(rootView: View?) {
        this.rootView = rootView
    }

    override fun showSnackBar(@StringRes stringId: Int) {
        showSnackBar(activity.resources.getString(stringId), Snackbar.LENGTH_SHORT)
    }

    override fun showSnackBar(message: String) {
        showSnackBar(message, Snackbar.LENGTH_SHORT)
    }

    override fun showSnackBar(message: String, length: Int) {
        showSnackBar(message, SnackBarMessageType.DEFAULT, length)
    }

    override fun showSnackBar(message: String, type: SnackBarMessageType, length: Int) {
        snack?.dismiss()
        snack = Snackbar
            .make(parentView, message, length)
//            .setAction(actionText(type)) {}
//            .setActionTextColor(color(type))

        Timber.w(" act - ${activity} ")
        Timber.w(" act - ${(activity as? IBottomNavigation)} ")
//        (activity as? IBottomNavigation)?.let {bNAv ->
//            Timber.w(" act $bNAv" )
//            snack?.apply {
//                Timber.w(" sna")
//                view.layoutParams = (view.layoutParams as CoordinatorLayout.LayoutParams).apply {
//                    Timber.w(" fusc $this")
//                    setMargins(
//                        leftMargin,
//                        topMargin,
//                        rightMargin,
//                        300
//                    )
//                }
//            }?.show()
//        }

        snack?.show()
    }
}

interface ISnackBarProvider{
    fun bindRootView(rootView : View?)
    val isRootBounded : Boolean
    val snackBar : Snackbar?
    fun showSnackBar(@StringRes stringId: Int)
    fun showSnackBar(message: String)
    fun showSnackBar(message: String, length: Int)
    fun showSnackBar(message: String, type: SnackBarMessageType, length: Int)
}

enum class SnackBarMessageType{
    DEFAULT,
    ERROR,
    ATTENTION,
    OK
}